'''implement NUBISCAN like motif searches.'''
# import numpy
import math, re, sys
import collections, bisect
import CGAT.Stats as Stats

cimport cython

# note both import and cimport are necessary
import numpy
cimport numpy
DTYPE_INT = numpy.int
ctypedef numpy.int_t DTYPE_INT_t
DTYPE_FLOAT = numpy.float
ctypedef numpy.float_t DTYPE_FLOAT_t

@cython.boundscheck(False) # turn of bounds-checking for entire function
def computeScores( numpy.ndarray[DTYPE_INT_t, ndim=1]seq,
                   numpy.ndarray[DTYPE_FLOAT_t, ndim=2]matrix,
                   numpy.ndarray[DTYPE_FLOAT_t, ndim=1]weights ):
    '''return a score vector. Each position in seq is scored
    against the matrix.
    
    The matrix should be sorted by ACGT (rows) and motif positions
    (columns).

    The sequence is a numeric vector.
    '''

    assert seq.dtype == DTYPE_INT and matrix.dtype == DTYPE_FLOAT and weights.dtype == DTYPE_FLOAT
    cdef int lalphabet = matrix.shape[0]
    cdef int lmotif = matrix.shape[1]
    cdef int l = len(seq) - lmotif
    cdef numpy.ndarray[ DTYPE_FLOAT_t, ndim=1] result = numpy.zeros( l, dtype = DTYPE_FLOAT )
    cdef DTYPE_FLOAT_t score = 0
    cdef int x, y, i
    # nested loop implementation, could made more elegant and quicker using
    # numpy annotation? Convert to cython?
    for 0 <= x < l:
        score = 0
        i = 0
        for x <= y < x+lmotif:
            score += weights[i] * matrix[seq[y],i]
            i += 1
        result[x] = score

    return result

@cython.boundscheck(False) # turn of bounds-checking for entire function
def combineScores(
    numpy.ndarray[ DTYPE_FLOAT_t, ndim=1 ]sense_scores,
    numpy.ndarray[ DTYPE_FLOAT_t, ndim=1 ]antisense_scores,
    int lmotif,
    arrangement = "DR3" ):

    assert sense_scores.dtype == DTYPE_FLOAT and antisense_scores.dtype == DTYPE_FLOAT
    cdef int distance, x, y
    cdef int l = len(sense_scores)
        
    orientation, distance = arrangement[:2], int(arrangement[2:])
    distance += lmotif

    cdef numpy.ndarray[ DTYPE_FLOAT_t, ndim=1] sense_results = numpy.zeros( l, dtype = DTYPE_FLOAT )
    cdef numpy.ndarray[ DTYPE_FLOAT_t, ndim=1] antisense_results = numpy.zeros( l, dtype = DTYPE_FLOAT )
    
    if orientation == "DR":
        for 0 <= x < l - distance:
#             sense_results[x] = (sense_scores[x] + sense_scores[x+distance]) / 2.0
#             antisense_results[x] = (antisense_scores[x] + antisense_scores[x+distance]) / 2.0
              sense_results[x] = min(sense_scores[x],sense_scores[x+distance])
              antisense_results[x] = min(antisense_scores[x],antisense_scores[x+distance])
    elif orientation == "ER":
        for 0 <= x < l - distance:
#             sense_results[x] = (sense_scores[x] + antisense_scores[x+distance] ) / 2.0
#             antisense_results[x] = (antisense_scores[x] + sense_scores[x+distance]) / 2.0
            sense_results[x] = min(sense_scores[x], antisense_scores[x+distance])
            antisense_results[x] = min(antisense_scores[x], sense_scores[x+distance])
    else:
        raise ValueError("invalid orientation `%s`" % orientation )
            
    return sense_results, antisense_results

def mapSequence( seq ):
    '''mapSequence to numbers.'''
    chars = "ACGTN"
    result = []
    for x in re.sub("\s", "", seq).upper(): result.append( chars.find( x ) )
    assert min(result) >= 0
    return numpy.array( result, dtype=numpy.int )

def computeWeights( matrix ):
    '''compute weights from matrix.

    The formula in the NUBISCAN paper is Wi = 100 / ln(4) * ( sum_over_ACGT (pa * log(pa)) + ln(4) )
    '''

    lalphabet, lmotif = matrix.shape
    # ignore dummy row for N
    lalphabet -= 1
    g = math.log(lalphabet)
    weights = numpy.zeros( lmotif, numpy.float )
    for y in range( lmotif ):
        w = 0
        for x in range(lalphabet):
            v = matrix[x,y]
            if v != 0: i =  v * math.log( v ) 
            else: i = 0
            w += i 
        weights[y] = 100.0 / g * (w + g) 
    return weights

# first half site for rxrvdr
#      0.020000  0.100000  0.020000  0.860000
#      0.160000  0.000000  0.840000  0.000000
#      0.820000  0.120000  0.040000  0.020000
#      0.660000  0.340000  0.000000  0.000000
#      0.000000  1.000000  0.000000  0.000000
#      0.000000  0.320000  0.000000  0.680000
# second half site for rxrvdr (not used)
#     0.020000  0.000000  0.000000  0.980000
#     0.040000  0.040000  0.920000  0.000000
#     0.920000  0.020000  0.060000  0.000000
#     0.260000  0.740000  0.000000  0.000000
#     0.000000  1.000000  0.000000  0.000000
#     0.200000  0.380000  0.320000  0.100000

def getAntisenseMatrix( matrix, weights = None ):
    
    as_matrix = numpy.vstack( (matrix[3,::-1],
                               matrix[2,::-1],
                               matrix[1,::-1],
                               matrix[0,::-1],
                               matrix[4,::-1], ) )
    
    if weights != None:
        return as_matrix, weights[::-1]
    else:
        return as_matrix
    
class Matcher(object):

    def __init__(self, matrix, *args, **kwargs ):
        self.matrix = matrix
        self.weights = computeWeights( matrix )

        self.antisense_matrix, self.antisense_weights = \
                               getAntisenseMatrix( matrix,
                                                   self.weights )

        max_scores = numpy.array( self.matrix.max( 0 ).flat)
        self.max_score = numpy.dot( self.weights, max_scores)

    def run( self, sequences, arrangements):
        pass

    def parseArrangement( self, arrangement ):
        '''parse arrangment into tuple of orientation, distance.'''
        orientation, distance = arrangement[:2], int(arrangement[2:])
        return orientation, distance
    
NubiscanMatch = collections.namedtuple ("NubiscanMatch",
                                        '''id, length, start, end, strand,
                                        arrangement, 
                                        score, zscore, pvalue, qvalue,
                                        alternatives''' )
    
class Nubiscan( Matcher ):
    '''Find motif using the standard NUBISCAN procedure.

    Motifs are filtered using a z-score threshold.

    Z-scores are compute individually for each repeat arrangement.
    '''

    def run( self, sequences, arrangements, zscore_threshold = 6.5 ):

        lalphabet, lmotif = self.matrix.shape
        assert lalphabet == 5

        sense_matrix = self.matrix
        sense_weights = self.weights

        antisense_matrix, antisense_weights = self.antisense_matrix, \
                                              self.antisense_weights
                 
        max_scores = numpy.array( sense_matrix.max( 0 ).flat)
        max_score = numpy.dot( sense_weights, max_scores)

        # compute all scores for all sequence positions
        sense_halfsite_scores, antisense_halfsite_scores = [], []
        for sequence in sequences:
            s = mapSequence(sequence)
            sense_halfsite_scores.append( computeScores( s, sense_matrix, sense_weights ) / max_score )
            antisense_halfsite_scores.append( computeScores( s, antisense_matrix, antisense_weights ) / max_score )

        results = []                

        # iterate over arrangements
        for arrangement in arrangements:

            orientation, distance = arrangement[:2], int(arrangement[2:])
            # offset to end
            distance += 2 * lmotif 

            sense_combined_scores, antisense_combined_scores, all_scores = [], [], []
            for s_hs, as_hs in zip( sense_halfsite_scores, antisense_halfsite_scores ):
                s_cs, as_cs = combineScores(
                    s_hs,
                    as_hs,
                    lmotif,
                    arrangement )
                sense_combined_scores.append( s_cs )
                antisense_combined_scores.append( as_cs )

            all_scores = numpy.concatenate( sense_combined_scores + antisense_combined_scores )
            mean = numpy.mean( all_scores)
            std = numpy.std( all_scores )

            for x, scores in enumerate( sense_combined_scores ):
                for start, score in enumerate( scores ):
                    zscore = (score - mean) / std
                    if zscore >= zscore_threshold:
                        results.append( NubiscanMatch._make( (x,
                                                              0,
                                                              start,
                                                              start + distance,
                                                              "+",
                                                              arrangement,
                                                              score,
                                                              zscore,
                                                              0, 0, None
                                                               ) ) )

            for x, scores in enumerate( antisense_combined_scores ):
                for start, score in enumerate( scores ):
                    zscore = (score - mean) / std
                    if zscore >= zscore_threshold:
                        results.append( NubiscanMatch._make( (x,
                                                              start,
                                                              start + distance,
                                                              "-",
                                                              arrangement,
                                                              score,
                                                              zscore,
                                                              0, 0, None
                                                              ) ) )

        return results
        
class MatcherRandomisationSequence( Matcher ):
    '''Find motifs using the NUBISCAN procedure for a single sequence.

    Results are filtered using a FDR procedure.

    The procedure is as follows:

    The sequence is randomised x times and motif scores for
    all arrangements and locations are collected. From these scores an
    empirical P-Value distribution is derived. The number of samples
    can be small, as the number of positions is likely to be large.
    This randomisation ensures that sequences 

    Motif scores are computed for all arrangements and possible locations
    within the original sequence. Each motif score is assigned a pvalue based
    on the randomized positions.

    From the pvalues for motif scores for the original sequence, qvalues
    are computed using the method from Storey et al. (2002).

    Only values less than the q-value threshold are returned.
    '''

    def __init__(self, matrix, samples = 100, *args, **kwargs ):
        Matcher.__init__( self, matrix, *args, **kwargs )
        self.samples = samples
        self.binsize = 0.001
        
    def computeBackground( self, seq, arrangements ):
        '''randomise sequences and compute P-Values for scores.

        return an array of P-Values.
        '''
        
        lalphabet, lmotif = self.matrix.shape
        # create a copy
        r = numpy.array(seq)
        lseq = len(seq)
        # nbins = int(math.ceil(1.0 / self.binsize)) + 1
        bg_scores = []
        t = 0
        for x in range( self.samples ):
            numpy.random.shuffle(r)
            # compute sense(s)/antisense(a) halfsite (hs) scores
            s_hs, a_hs = computeScores( r, self.matrix, self.weights ) / self.max_score, \
                         computeScores( r, self.antisense_matrix, self.antisense_weights ) / self.max_score

            for arrangement in arrangements:
                s_cs, as_cs = combineScores(
                    s_hs,
                    a_hs,
                    lmotif,
                    arrangement )
                # for x in s_cs: hist[int(x/self.binsize)] += 1
                # for x in as_cs: hist[int(x/self.binsize)] += 1
                
                # bg_scores.extend( s_cs )
                # bg_scores.extend( as_cs )
                bg_scores.append( s_cs )
                bg_scores.append( as_cs )
                
        #bg_scores = numpy.array( bg_scores )
        #bg_mean = numpy.mean( bg_scores)
        #bg_std = numpy.std( bg_scores )
        #t = len(bg_scores)
        hist, bin_edges = numpy.histogram( numpy.concatenate(bg_scores),
                                           bins = numpy.arange(0,1.0+2*self.binsize,self.binsize),
                                           normed = False, new = True )
        bg_mean = 0.5
        bg_std = 0.1
        t = sum(hist)
        # t = lseq * self.samples * 2 
        pvalues = (hist[::-1].cumsum() / float(t))[::-1]
        return pvalues, t, bg_mean, bg_std

    def collectMatches( self, seq, arrangements, pvalues ):
        '''compute matching score and pvalue for each site and arrangement.
        '''

        lalphabet, lmotif = self.matrix.shape
        sense_halfsite_scores, anti_halfsite_scores = [], []
        s_hs = computeScores( seq, self.matrix, self.weights ) \
               / self.max_score 
        a_hs = computeScores( seq, self.antisense_matrix, self.antisense_weights ) \
               / self.max_score 

        results = []
        # iterate over arrangements
        append = results.append
        for arrangement in arrangements:

            s_cs, as_cs = combineScores( s_hs,
                                         a_hs,
                                         lmotif,
                                         arrangement )
            
            for start, score in enumerate( s_cs ):
                pvalue = pvalues[ int(score / self.binsize) ]
                append( (arrangement, start, "+", pvalue, score ) )
                
            for start, score in enumerate( as_cs ):
                pvalue = pvalues[ int(score / self.binsize) ]
                append( (arrangement, start, "-", pvalue, score ) )

        return results
        
    def run( self, sequence, arrangements, qvalue_threshold = None):
        
        lalphabet, lmotif = self.matrix.shape
        assert lalphabet == 5

        seq = mapSequence(sequence)

        # do the sampling
        bg_pvalues, t, bg_mean, bg_std = self.computeBackground( seq, arrangements )

        # compute all matches
        matches = self.collectMatches( seq, arrangements, bg_pvalues )

        if qvalue_threshold != None:
            # collect pvalues
            pvalues = [ x[3] for x in matches ]

            # estimate qvalues for matches
            fdr = Stats.doFDR( pvalues )
            qvalues = fdr.mQValues
        else:
            qvalues = [0] * len(matches)
            qvalue_threshold = 1

        results = []
        # filter by qvalue
        for 0 <= x < len(matches):
            match = matches[x]
            qvalue = qvalues[x]
            if qvalue > qvalue_threshold: continue
            arrangement, start, strand, pvalue, score = match
            orientation, distance = self.parseArrangement( arrangement )
            zscore = (score - bg_mean) / bg_std
            results.append( NubiscanMatch._make( ("",
                                                  start,
                                                  start + distance + 2*lmotif,
                                                  strand,
                                                  arrangement,
                                                  score,
                                                  zscore,
                                                  pvalue,
                                                  qvalue,
                                                  None
                                                  ) ) )
        
        return results

class MatcherRandomisationSequences( MatcherRandomisationSequence ):
    '''Find motifs using the NUBISCAN procedure for multiple sequences.

    Results are filtered using a FDR procedure.

    The procedure is as follows:

    The sequence is randomised x times and motif scores for
    all arrangements and locations are collected. From these scores an
    empirical P-Value distribution is derived. The number of samples
    can be small, as the number of positions is likely to be large.
    This randomisation ensures that sequences 

    Motif scores are computed for all arrangements and possible locations
    within the original sequence. Each motif score is assigned a pvalue based
    on the randomized positions.

    From the pvalues for motif scores for the original sequence, qvalues
    are computed using the method from Storey et al. (2002).

    Only values less than the q-value threshold are returned.
    '''

    def __init__(self, matrix, samples = 100, *args, **kwargs ):
        Matcher.__init__( self, matrix, *args, **kwargs )
        self.samples = samples
        self.binsize = 0.001
        
    def collectMatches( self, seq, arrangements ):
        '''compute matching score and pvalue for each site and arrangement.
        '''

        lalphabet, lmotif = self.matrix.shape
        sense_halfsite_scores, anti_halfsite_scores = [], []
        s_hs = computeScores( seq, self.matrix, self.weights ) \
               / self.max_score
        a_hs = computeScores( seq, self.antisense_matrix, self.antisense_weights ) \
               / self.max_score 

        results = []
         
        # iterate over arrangements
        append = results.append
        for arrangement in arrangements:

            s_cs, as_cs = combineScores( s_hs,
                                         a_hs,
                                         lmotif,
                                         arrangement )

            append( (arrangement, s_cs, as_cs ) )

        return results
        
    def run( self,
             sequences,
             arrangements,
             qvalue_threshold = 0.05,
             pvalue_threshold = 0.01 ):
        
        lalphabet, lmotif = self.matrix.shape
        assert lalphabet == 5

        # do the sampling per sequence
        all_pvalues = []
        
        # keep only matches below a certain threshold -
        # otherwise there are too many.
        putative_matches = []
        binsize = self.binsize
        append = all_pvalues.append
        cdef int start, l
        cdef float log_pvalue_threshold = numpy.log(pvalue_threshold)

        for seq_id, seq in enumerate(sequences):

            seq = mapSequence(seq)
            
            bg_pvalues, t, bg_mean, bg_std = self.computeBackground( seq, arrangements )

            # convert pvalues to logspace
            # minimum P-value: number of values
            bg_pvalues = numpy.log( bg_pvalues )
            min_p = 1.0 / t
            bg_pvalues[ numpy.isneginf(bg_pvalues) ] = numpy.log( min_p )
            # compute all matches
            matches = self.collectMatches( seq, arrangements )
        
            for arrangement, sense_scores, anti_scores in matches:
                l = len(sense_scores)
                
                # compute p-values (in log-space)
                sense_p = numpy.array( [ bg_pvalues[int(x/binsize)] for x in sense_scores ] )
                anti_p = numpy.array( [ bg_pvalues[int(x/binsize)] for x in anti_scores ] )
                
                all_pvalues.append( sense_p )
                all_pvalues.append( anti_p )
                
                # record putative matches:
                for 0 <= start < l:
                    if sense_p[start] < log_pvalue_threshold:
                        putative_matches.append( (arrangement, seq_id, start, "+", sense_scores[start], sense_p[start]) )
                    if anti_p[start] < log_pvalue_threshold:
                        putative_matches.append( (arrangement, seq_id, start, "-", anti_scores[start], anti_p[start]) )

        # there are too many p-values to compute q-values for all
        # instead - work within a histogram.
        #
        # compute empirical qvalues based on all pvalues
        all_pvalues = numpy.concatenate(all_pvalues)
        hist, bin_edges = numpy.histogram( all_pvalues,
                                           bins=10000,
                                           normed = False,
                                           new = True )

        # qvalue = pi0 * t * pvalue / {number of observations <= pvalue}
        t = sum(hist)
        pi0 = Stats.getPi0( numpy.exp(all_pvalues) )
        # factor in pi0 to nobservations
        nobservations = hist.cumsum() / float(t) / pi0

        results = []
        # filter by qvalue
        nq = len(nobservations) -1
        for 0 <= x < len(putative_matches):
            match = putative_matches[x]
            arrangement, seq_id, start, strand, score, log_pvalue = match
            pvalue = math.exp(log_pvalue)
            # number of observations <= pvalue
            nobs = nobservations[ min(bisect.bisect( bin_edges, log_pvalue), nq) ]
            qvalue = pvalue / nobs
            if qvalue > qvalue_threshold: continue
            
            orientation, distance = self.parseArrangement( arrangement )
            zscore = (score - bg_mean) / bg_std
            results.append( NubiscanMatch._make( (seq_id,
                                                  len(sequences[seq_id]),
                                                  start,
                                                  start + distance + 2*lmotif,
                                                  strand,
                                                  arrangement,
                                                  score,
                                                  zscore,
                                                  pvalue,
                                                  qvalue,
                                                  None
                                                  ) ) )

        # save results for calls
        self.bin_edges = bin_edges
        self.hist = hist
        self.nobservations = nobservations
        self.pi0 = pi0
        
        return results




################################################################################
#   Gene prediction pipeline 
#
#   $Id: diff_gtfs.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2004 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
diff_gtfs.py - compute overlap between multiple gtf files
=========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script compares multiple set of gtf files. It computes
the overlap between bases, exons and genes between each pair
of gtf files. 

If results from a previous run are present, existing
pairs are not re-computed but simply echoed.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import string
import re
import optparse
import collections
import numpy

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.NCL as NCL

class Counter:

    mPercentFormat = "%5.2f"

    def __init__(self):
        pass
        
    def getHeader( self ):
        h = []
        for a in ("genes", "exons", "bases" ):
            for b in ("total", "ovl", "unique"):
                for c in ("1","2"):
                    h.append( "n" + a + "_" + b + c )
        for a in ("genes", "exons", "bases" ):
            for b in ("ovl", "unique"):
                for c in ("1","2"):
                    h.append( "p" + a + "_" + b + c )

        return "\t".join(h)


    @E.cachedmethod
    def buildIndex( self, filename ):
        """read and index."""

        idx = {}
        infile = IOTools.openFile(filename, "r")
        for e in GTF.readFromFile(infile):
            if e.contig not in idx: idx[e.contig] = ncl.NCLSimple()
            idx[e.contig].add( e.start,e.end )
        infile.close()
        return idx

    def _count( self, filename, idx ):

        overlapping_genes = set()
        genes = set()
        # iterate over exons
        infile = IOTools.openFile( filename, "r" )
        it = GTF.iterator( infile )

        nexons, nexons_overlapping = 0, 0
        nbases, nbases_overlapping = 0, 0
        for this in it:
            nexons += 1
            nbases += this.end - this.start 
            genes.add( this.gene_id )
            
            try:
                intervals = list(idx[this.contig].find( this.start, this.end ))
            except KeyError:
                continue
            
            if len(intervals) == 0: 
                continue

            overlapping_genes.add( this.gene_id )
            nexons_overlapping += 1
            start, end = this.start, this.end 
            counts = numpy.zeros( end - start, numpy.int )
            for other_start,other_end,other_value in intervals:
                for x in range( max(start, other_start ) - start, min(end, other_end ) - start ): 
                    counts[x] += 1
            nbases_overlapping += sum( [1 for x in counts if x > 0 ] )

        infile.close()

        return len(genes), len( overlapping_genes), nexons, nexons_overlapping, nbases, nbases_overlapping

    def count( self, filename1, filename2 ):
        """count overlap between two gtf files."""

        E.info( "counting started for %s versus %s" % (filename1, filename2))

        idx2 = self.buildIndex( filename2 )

        (self.mGenes1, self.mGenesOverlapping1,
         self.mExons1, self.mExonsOverlapping1,
         self.mBases1, self.mBasesOverlapping1 ) = \
         self._count( filename1, idx2 )

        self.mGenesUnique1 = self.mGenes1 - self.mGenesOverlapping1
        self.mExonsUnique1 = self.mExons1 - self.mExonsOverlapping1
        self.mBasesUnique1 = self.mBases1 - self.mBasesOverlapping1

        idx1 = self.buildIndex( filename1 )

        (self.mGenes2, self.mGenesOverlapping2,
         self.mExons2, self.mExonsOverlapping2,
         self.mBases2, self.mBasesOverlapping2 ) = \
         self._count( filename2, idx1 ) 

        self.mGenesUnique2 = self.mGenes2 - self.mGenesOverlapping2
        self.mExonsUnique2 = self.mExons2 - self.mExonsOverlapping2
        self.mBasesUnique2 = self.mBases2 - self.mBasesOverlapping2

    def __str__(self):

        return "\t".join( map( str, (
                        self.mGenes1, self.mGenes2,
                        self.mGenesOverlapping1, self.mGenesOverlapping2,
                        self.mGenesUnique1, self.mGenesUnique2,
                        self.mExons1, self.mExons2,
                        self.mExonsOverlapping1, self.mExonsOverlapping2,
                        self.mExonsUnique1, self.mExonsUnique2,
                        self.mBases1, self.mBases2,
                        self.mBasesOverlapping1, self.mBasesOverlapping2,
                        self.mBasesUnique1, self.mBasesUnique2 ) ) ) + "\t" +\
              "\t".join( map( lambda x: IOTools.prettyPercent( *x), (
                    (self.mGenesOverlapping1, self.mGenes1),
                    (self.mGenesOverlapping2, self.mGenes2),
                    (self.mGenesUnique1, self.mGenes1), 
                    (self.mGenesUnique2, self.mGenes2),
                    (self.mExonsOverlapping1, self.mExons1), 
                    (self.mExonsOverlapping2, self.mExons2),
                    (self.mExonsUnique1, self.mExons1), 
                    (self.mExonsUnique2, self.mExons2),
                    (self.mBasesOverlapping1, self.mBases1), 
                    (self.mBasesOverlapping2, self.mBases2),
                    (self.mBasesUnique1, self.mBases1), 
                    (self.mBasesUnique2, self.mBases2)  ) ) )

class CounterGenes(Counter):
    """output only genes."""
    
    mSeparator = ";"

    def __init__(self, *args, **kwargs ):
        Counter.__init__(self, *args, **kwargs)

    def getHeader( self ):
        h = [ "ngenes1", "ngenes2", "novl1", "novl2", "nuniq1", "nuniq2", "ovl1", "ovl2", "uniq1", "uniq2" ]
        return "\t".join(h)

    def _count( self, filename, idx ):

        overlapping_genes = set()
        genes = set()
        # iterate over exons
        infile = IOTools.openFile( filename, "r" )
        it = GTF.iterator( infile )

        for this in it:
            genes.add( this.gene_id )
            
            try:
                intervals = idx[this.contig].find( this.start, this.end )
            except KeyError:
                continue

            if len(intervals) == 0: 
                continue

            overlapping_genes.add( this.gene_id )

        infile.close()

        return genes, overlapping_genes

    def count( self, filename1, filename2 ):
        """count overlap between two gtf files."""

        E.info( "counting started for %s versus %s" % (filename1, filename2))

        idx2 = self.buildIndex( filename2 )

        (self.mGenes1, self.mGenesOverlapping1 ) =  self._count( filename1, idx2 )

        idx1 = self.buildIndex( filename1 )
        (self.mGenes2, self.mGenesOverlapping2) = self._count( filename2, idx1 )    

    def __str__(self):

        uniq1 = self.mGenes1.difference(self.mGenesOverlapping)
        uniq2 = self.mGenes2.difference(self.mGenesOverlapping)
        
        return "\t".join( map(str, ( \
                    len(self.mGenes1),
                    len(self.mGenes2),
                    len(self.mGenesOverlapping1),
                    len(self.mGenesOverlapping2),
                    len(uniq1),
                    len(uniq2),
                    self.mSeparator.join( self.mGenesOverlapping1),
                    self.mSeparator.join( self.mGenesOverlapping2),
                    self.mSeparator.join( uniq1 ), 
                    self.mSeparator.join( uniq2 ) ) ) )

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: diff_gtfs.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-s", "--ignore-strand", dest="ignore_strand", action="store_true",
                      help="ignore strand information [default=%default]." )

    parser.add_option("-u", "--update", dest="filename_update", type="string",
                      help="if filename is given, previous results will be read from there and only changed sets will be computed [default=%default]." )

    parser.add_option("-p", "--pattern-id", dest="pattern_id", type="string",
                      help="pattern to convert a filename to an id [default=%default]." )

    parser.add_option("-g", "--only-genes", dest="only_genes", action="store_true",
                      help="only output gene stats (includes gene lists) [default=%default]." )

    parser.set_defaults(
        ignore_strand = False,
        filename_update = None,
        pattern_id = "(.*).gtf",
        only_genes = False,
        )

    (options, args) = E.Start( parser )

    if len(args) < 2:
        print USAGE
        raise ValueError( "at least two arguments required" )

    if options.filename_update:
        infile = open( options.filename_update, "r")
        previous_results = {}
        for line in infile:
            if line.startswith("#"): continue
            if line.startswith("set1"): continue
            data = line[:-1].split("\t")
            set1,set2 = data[0],data[1]

            if set1 not in previous_results:
                previous_results[set1] = {}
            if set2 not in previous_results:
                previous_results[set2] = {}

            previous_results[set1][set2] = "\t".join( data[2:] )
            rev = [ (data[x+1], data[x]) for x in range( 2, len(data), 2 ) ]
            previous_results[set2][set1] = "\t".join( IOTools.flatten(rev) ) 
    else:
        previous_results = {}

    if options.only_genes:
        counter = CounterGenes()
    else:
        counter = Counter()

    options.stdout.write( "set1\tset2\t%s\n" % counter.getHeader()  )

    pattern_id = re.compile(options.pattern_id)

    def getTitle( x ):
        try:
            return pattern_id.search( x ).groups()[0]
        except AttributeError:
            return x

    ncomputed, nupdated = 0, 0
    for x in range( len(args)):
        title1 = getTitle( args[x] )
        for y in range( 0,x):
            title2 = getTitle( args[y] )
            if previous_results:
                try:
                    prev = previous_results[title1][title2] 
                except KeyError:
                    pass
                else:
                    options.stdout.write( "%s\t%s\t%s\n" % ( (title1, title2, prev ) ))
                    nupdated += 1
                    continue

            counter.count( args[x], args[y] )
            options.stdout.write( "%s\t%s\t%s\n" % ( (title1, title2, str(counter) ) ))
            ncomputed += 1

    E.info( "nupdated=%i, ncomputed=%i" % (nupdated, ncomputed))

    E.Stop()    



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <andreas.heger@helsinki.fi>
##
## $Id: psl2gff.py 2781 2009-09-10 11:33:14Z andreas $
##
##
####
####
'''
psl2gff.py - convert from psl to gff
====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script converts a :term:`gff` formatted
file to a :term:`psl` formatted file.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import re
import string
import optparse
import time
import os
import glob

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Blat as Blat
import CGAT.GTF as GTF

def main():

    parser = E.OptionParser( version = "%prog version: $Id: psl2gff.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-a", "--as-gtf", dest="as_gtf", action="store_true",
                      help="output as gtf."  )

    parser.add_option("-s", "--filename-strand", dest="filename_strand", type="string",
                      help="set strand information according to file [default=%DEFAULT]."  )

    parser.set_defaults( as_gtf = False,
                         filename_strand = None,
                         test = None )
    
    (options, args) = E.Start( parser, add_pipe_options = True )

    ####################################
    if options.filename_strand:
        map_id2strand = IOTools.readMap( open(options.filename_strand,"r")) 
    else:
        map_id2strand = {}

    iterator = Blat.BlatIterator( sys.stdin )

    ninput, noutput, nskipped = 0, 0, 0

    if options.as_gtf:
        gff = GTF.Entry()
    else:
        gff = GTF.Entry()

    gff.source = "psl"
    gff.feature = "exon"

    ids = {}

    while 1:
        
        if options.test and ninput >= options.test:
            break

        match = iterator.next()
        
        if match == None: break

        ninput += 1

        if match.mQueryId not in ids:
            ids[match.mQueryId] = 1
            id = match.mQueryId
        else:
            id = match.mQueryId + ":%i" % ids[match.mQueryId]
            ids[match.mQueryId] += 1

        if options.as_gtf:
            gff.contig = match.mSbjctId
            gff.gene_id = id
            gff.transcript_id = id
        else:
            gff.contig = match.mSbjctId
            gff.clearAttributes()
            gff.addAttribute( "gene_id", id )

        if id in map_id2strand:
            gff.strand = map_id2strand[id]
        else:
            gff.strand = match.strand

        for qstart, sstart, size in match.getBlocks():
            
            gff.start = sstart
            gff.end = sstart + size
            options.stdout.write( str( gff ) + "\n" )
        
        noutput += 1

    E.info("ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput, nskipped) )

    E.Stop()

if __name__ == '__main__':
    sys.exit(main())



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperBl2Seq.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse

"""Wrapper for adaptive codon bias program
"""

import Experiment
import FastaIterator

class Bl2SeqError(Exception):
    pass
            
class Bl2Seq:

    mOptions = ""
    mExecutable = "bl2seq"
    mStderr = sys.stderr
    
    def __init__( self, options=""):
        
        self.mOptions = options

    def CreateTemporaryFiles( self ):
        """create temporary files."""
        self.mTempDirectory = tempfile.mkdtemp()
        
        self.mFilenameTempInput = self.mTempDirectory + "/input"
        self.mFilenameTempOutput = self.mTempDirectory + "/output"

    def DeleteTemporaryFiles( self ):
        """clean up."""
        os.remove( self.mFilenameTempInput )
        os.remove( self.mFilenameTempOutput )
        os.rmdir( self.mTempDirectory )

    def SetStderr( self, file = None):
        """set file for dumping stderr."""
        self.mStderr= file
        
    def WriteOutput( self, lines, filename_output = None):
        """write output to file.

        If file is not given, lines are written to stdout.
        """
        
        if filename_output:
            outfile = open(filename_output, "w")
        else:
            outfile = sys.stdout
            
        outfile.write( string.join( lines, "") )

        if filename_output:
            outfile.close()

    def ParseResult( self, trace_file = None, information_file = None ):

        result = AdaptiveCAIResult()

        result.Read( trace_file, information_file )
        
        return result
    
    def RunOnFile( self, infile, outfile, errfile ):

        self.CreateTemporaryFiles()

        statement = string.join( ( self.mExecutable,
                                   self.mFilenameTempInput,
                                   self.mFilenameTempOutput ),
                                 " ")

        
        i = FastaIterator.FastaIterator( infile )

        outfile.write( "GENE\tBl2Seq\n")
        
        while 1:
            f = i.next()
            if f == None: break

            file = open(self.mFilenameTempInput, "w")
            file.write( ">%s\n%s" % (f.title, f.sequence) )
            file.close()

            
            s = subprocess.Popen( statement,
                                  shell = True,
                                  stdout = subprocess.PIPE,
                                  stderr = subprocess.PIPE,
                                  cwd = self.mTempDirectory,
                                  close_fds = True)                              

            (out, err) = s.communicate()

            if s.returncode != 0:
                raise Bl2SeqError, "Error in calculating Bl2Seq\n%s" % err

            d = open(self.mFilenameTempOutput).readlines()[2][:-1]
            enc = d.split( " " )[2]
            
            outfile.write( (string.join((f.title, enc), "\t") )  + "\n")
            
            errfile.write( err )

        self.DeleteTemporaryFiles()
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: WrapperBl2Seq.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-f", "--input-file", dest="input_filename", type="string",
                      help="input filename. If '-', stdin is used [default=%default].",
                      metavar="FILE" )

    parser.add_option("-o", "--output-file", dest="output_filename", type="string",
                      help="output filename for codon usage. If '-', output is stdout [default=%default].",
                      metavar="FILE" )

    parser.add_option("-e", "--error-file", dest="error_filename", type="string",
                      help="output filename for error messages. If '-', output is stderr [default=%default].",
                      metavar="FILE" )


    parser.set_defaults( \
        input_filename = "-",
        output_filename = "-",
        error_filename = "/dev/null",
        )
    
    (options, args) = Experiment.Start( parser ) 

    wrapper = Bl2Seq( )

    if options.input_filename == "-":
        file_stdin = sys.stdin
    else:
        file_stdin = open( options.input_filename, "r" )

    if options.output_filename:
        if options.output_filename == "-":
            file_stdout = sys.stdout
        else:
            file_stdout = open( options.output_filename, "w")

    if options.error_filename:
        if options.error_filename == "-":
            file_stderr = sys.stderr
        else:
            file_stderr = open( options.error_filename, "w")
        
    wrapper.RunOnFile( file_stdin, file_stdout, file_stderr )

    if file_stdin and file_stdin != sys.stdin:
        file_stdin.close()

    if file_stdout and file_stdout != sys.stdout:
        file_stdout.close()

    if file_stderr and file_stderr != sys.stderr:
        file_stderr.close()

    Experiment.Stop()
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
nofarm.py - stand-in for ``farm.py``
====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script is a stand-in for :doc:`farm`. Instead of running
splitting the input and running jobs on the cluster, the command
is executed as is.

This script accepts all the options of ``farm.py``, but ignores
most of them.

Usage
-----

Example::

   python nofarm.py --help

Type::

   python nofarm.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import string
import optparse
import time
import glob
import subprocess
import tempfile
import shutil

import farm

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import threadpool

##--------------------------------------------------------------------
def main():

    parser = farm.getOptionParser()

    (options, args) = E.Start( parser, 
                                        add_cluster_options = True )


    cmd = args[0]
    if len(args) > 1:
        cmd += " '" + "' '".join(args[1:]) + "'" 

    cmd = re.sub( "%DIR%", "", cmd )
    retcode = subprocess.call( cmd,
                               shell = True,
                               stdin = sys.stdin,
                               stdout = sys.stdout,
                               cwd = os.getcwd(),
                               close_fds = True)                              
    E.Stop()

if __name__ == '__main__':
    main()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
bed.plot.py - plot 
=============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Create genomic plots in a set of intervals.

Methods implemented

igv 
   plot through an active IGV instance. 

Usage
-----

Example::

   python bed2plot.py < in.bed 

Type::

   python script_template.py --help

for command line help.

Documentation
-------------

Command line options
--------------------

'''

import os
import sys
import re
import glob
import sqlite3

import CGAT.IGV as IGV
import CGAT.Bed as Bed

import CGAT.Experiment as E

def main( argv = sys.argv ):

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id$", usage = globals()["__doc__"] )

    parser.add_option( "-m", "--method", dest="method", type="choice",
                       choices = ("igv",),
                       help = "method to create plots with [%default]" )

    parser.add_option( "-d", "--snapshot-dir", dest="snapshotdir", type = "string",
                       help = "directory to save snapshots in [%default]" )

    parser.add_option( "-f", "--format", dest="format", type="choice",
                       choices = ("png", "eps", "svg"),
                       help = "output file format [%default]" )

    parser.add_option( "-o", "--host", dest="host", type = "string",
                       help = "host that IGV is running on [%default]" )

    parser.add_option( "-p", "--port", dest="port", type = "int",
                       help = "port that IGV listens at [%default]" )

    parser.add_option( "-e", "--extend", dest="extend", type = "int",
                       help = "extend each interval by a number of bases [%default]" )

    parser.add_option( "-x", "--expand", dest="expand", type = "float",
                       help = "expand each region by a certain factor [%default]" )

    parser.set_defaults(
        method = "igv",
        host = '127.0.0.1',
        port = 61111,
        snapshotdir = os.getcwd(),
        extend = 0,
        format = "png",
        expand = 1.0,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    E.info( "connection to session on %s:%s" % (options.host, options.port) )

    E.info( "saving images in %s" % options.snapshotdir)
    igv = IGV.IGV( host = options.host,
                   port = options.port,
                   snapshot_dir = os.path.abspath( options.snapshotdir ) )

    c = E.Counter()
    for bed in Bed.iterator(options.stdin):

        c.input += 1

        # IGV can not deal with white-space in filenames
        name = re.sub("\s", "_", bed.name)
        
        E.info( "going to %s:%i-%i for %s" % (bed.contig, bed.start, bed.end, name))

        start, end = bed.start, bed.end
        extend = options.extend
        if options.expand:
            d = end - start
            extend = max( extend, (options.expand * d - d) // 2 )

        start -= extend
        end += extend

        igv.go( "%s:%i-%i" % (bed.contig, start, end ))

        fn = "%s.%s" % (name, options.format)
        E.info( "writing snapshot to '%s'" % fn )
        igv.save( fn )
    
        c.snapshots +=1 

    E.info( c )
    E.Stop()

if __name__ == "__main__":
    sys.exit( main() )



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: gff2bed.py 2861 2010-02-23 17:36:32Z andreas $
##
##
####
####
'''
gff2bed.py - convert from gff to bed
====================================

Purpose
-------

convert gff to bed formatted files.

Usage
-----

Type::

   python gff2bed.py --help

for command line usage.

Code
----
'''

import sys
import re
import string
import optparse
import time
import os
import itertools
import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Bed as Bed

def main( argv = sys.argv ):

    parser = E.OptionParser( version = "%prog version: $Id: gff2bed.py 2861 2010-02-23 17:36:32Z andreas $", 
                                    usage=globals()["__doc__"])
    
    parser.add_option( "--is-gtf", dest="is_gtf", action="store_true",
                       help="input file is gtf [default=%default] ")

    # parser.add_option( "--format", dest="format", type="choice",
    #                    help = "bed output format [%default]",
    #                    choices = ("bed3", "bed4", "bed6" ) )

    parser.add_option( "--name", dest="name", type="choice",
                       help = "field to use as the name field [%default]",
                       choices = ("gene_id", "transcript_id", "class", "family", "feature", "source" ) )
    
    parser.add_option( "--track", dest="track", type="choice",
                       choices=("feature", "source", None),
                       help="use feature/source field to define tracks [default=%default] ")

    parser.set_defaults(
        track = None,
        # format = "bed4",
        name = "gene_id",
        is_gtf = False )
    
    (options, args) = E.Start( parser, add_pipe_options = True )

    ninput, noutput = 0, 0
    
    is_gtf, name = options.is_gtf, options.name

    if is_gtf:
        iterator = GTF.iterator( options.stdin )
    else:
        iterator = GTF.iterator( options.stdin )

    if options.track:
        all_input = list( iterator )

        if options.track == "feature":
            grouper = lambda x: x.feature
        elif options.track == "source":
            grouper = lambda x: x.source

        all_input.sort( key = grouper )

        bed = Bed.Bed()
        for key, vals in itertools.groupby( all_input, grouper ):
            options.stdout.write( "track name=%s\n" % key )
            for gff in vals:
                ninput += 1
                bed.fromGTF( gff, is_gtf = is_gtf, name = name )
                options.stdout.write( str(bed) + "\n" )
                noutput += 1

    else:
        bed = Bed.Bed()
        for gff in iterator:
            ninput += 1
            bed.fromGTF( gff, is_gtf = is_gtf, name = name )
            options.stdout.write( str(bed) + "\n" )

            noutput += 1

    E.info( "ninput=%i, noutput=%i" % (ninput, noutput) )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv ) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: run_nubiscan.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""

:Author: Andreas Heger
:Release: $Id: run_nubiscan.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse
import numpy
import CGAT.Experiment as E
import CGAT.Nubiscan as Nubiscan
import CGAT.FastaIterator as FastaIterator
import CGAT.Stats as Stats
import CGAT.Genomics as Genomics
import CGAT.Masker as Masker

# from MEME
RXRVDR = numpy.matrix( (( 0.02,  0.16,  0.82,  0.66,  0.00,  0.00 ),
                        ( 0.10,  0.00,  0.12,  0.34,  1.00,  0.32 ),
                        ( 0.02,  0.84,  0.04,  0.00,  0.00,  0.00 ),
                        ( 0.86,  0.00,  0.02,  0.00,  0.00,  0.68 ),
                        (    0,     0,     0,     0,     0,     0 ),
                        ) )

# from Bioprospector - first half site
RXRVDR1 = numpy.matrix( ((  0.09,  0.22,  0.76,  0.70,  0.00,  0.00 ),
                        ( 0.15,  0.00,  0.17,  0.30,  1.00,  0.36 ),
                        ( 0.00,  0.77,  0.00,  0.00,  0.00,  0.00 ),
                        ( 0.76,  0.00,  0.07,  0.00,  0.00,  0.64 ),
                        (    0,     0,     0,     0,     0,     0 ),
                        ) )


# from Bioprospector - second half site
RXRVDR2 = numpy.matrix( ((  0.02,  0.02,  0.88,  0.27,  0.05,  0.03 ),
                        ( 0.05,  0.15,  0.07,  0.73,  0.92,  0.42 ),
                        ( 0.00,  0.83,  0.04,  0.00,  0.00,  0.00 ),
                        ( 0.93,  0.00,  0.01,  0.00,  0.04,  0.55 ),
                        (    0,     0,     0,     0,     0,     0 ),
                        ) )

# from Bioprospector - combined half-sites
RXVDR = (RXRVDR1 + RXRVDR2) / 2.0

NR =  numpy.matrix( (( 0.72,     0,  0.09,  0.27,     0,  0.90 ),
                     (    0,     0,     0,     0,  0.90,  0.09 ),
                     ( 0.27,  0.90,  0.54,  0.09,  0.09,     0 ),
                     (    0,  0.09,  0.36,  0.63,     0,     0 ),
                     (    0,     0,     0,     0,     0,     0 ),
                     ) )


def maskSequences( sequences, masker ):

    if masker == "repeatmasker":
        # the genome sequence is repeat masked
        masked_seq = sequences
    elif masker == "dust":
        masker_object = Masker.MaskerDustMasker()
        masked_seq = [ masker_object( x.upper() ) for x in sequences ]
    else:
        masked_seq = [x.upper() for x in sequences ]

    # hard mask softmasked characters
    masked_seq = [re.sub( "[a-z]","N", x) for x in masked_seq ]
    return masked_seq

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: run_nubiscan.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-i", "--iterations", dest="iterations", type="int",
                      help="number of iterations for sampling [default=%default]."  )

    parser.add_option("-q", "--qvalue", dest="qvalue_threshold", type="float",
                      help="qvalue threshold [default=%default]."  )

    parser.add_option("--without-combine", dest="combine", action = "store_false",
                      help="combine overlapping motifs [default=%default]."  )

    parser.add_option("-f", "--fdr-control", dest="fdr_control", type="choice",
                      choices = ("per-sequence", "all", "xall"),
                      help="qvalue threshold [default=%default]."  )

    parser.add_option("-m", "--motif", dest="motif", type="choice",
                      choices=("rxrvdr", "rxrvdr1", "rxrvdr2", "nr"),
                      help="qvalue threshold [default=%default]."  )

    parser.add_option("-a", "--arrangements", dest="arrangements", type="string",
                      help ="',' separated list of repeat arrangements [default=%default]")

    parser.add_option("-x", "--mask", dest="mask", type="choice",
                      choices=("dust","repeatmasker"),
                      help ="mask sequences before scanning [default=%default]")

    parser.add_option("--output-stats", dest="output_stats", action = "store_true",
                      help="output stats [default=%default]."  )

    parser.add_option("--add-sequence", dest="add_sequence", action = "store_true",
                      help="add sequence information [default=%default]."  )

    parser.set_defaults(
        iterations = 100,
        qvalue_threshold = 0.05,
        motif = "rxrvdr",
        fdr_control = "all",
        combine = True,
        arrangements = None,
        mask = None,
        output_stats = False,
        add_sequence = False,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    if options.arrangements == None:
        options.arrangements = [ "DR%s" % x for x in range(0,15) ] + [ "ER%s" % x for x in range(0,15) ]
    else:
        options.arrangements = options.arrangements.split(",")
        
    options.stdout.write( "%s" % "\t".join(Nubiscan.NubiscanMatch._fields) )
    if options.add_sequence: options.stdout.write( "\tsequence" )
    options.stdout.write("\n")

    if options.motif == 'nr': sense_matrix = NR
    elif options.motif == "rxrvdr": sense_matrix = RXRVDR
    elif options.motif == "rxrvdr1": sense_matrix = RXRVDR1
    elif options.motif == "rxrvdr2": sense_matrix = RXRVDR2
    else:
        raise ValueError("unknown matrix %s" % options.motif)

    if options.fdr_control == "all":

        seqs = list(FastaIterator.iterate(options.stdin))

        if options.mask:
            masked_seqs = maskSequences( [x.sequence for x in seqs], options.mask )
        else:
            masked_seqs = [x.sequence for x in seqs]
            
        ninput = len(seqs)
        map_id2title = dict( enumerate( [re.sub("\s.*", "", x.title) for x in seqs] ) )
        matcher = Nubiscan.MatcherRandomisationSequences( sense_matrix,
                                                          samples = options.iterations )
        
        results = matcher.run( masked_seqs,
                               options.arrangements,
                               qvalue_threshold = options.qvalue_threshold )

        if options.combine:
            results =  Nubiscan.combineMotifs( results )
        
        for r in results:

            if r.alternatives:
                alternatives = ",".join( [x.arrangement for x in r.alternatives ] )
            else:
                alternatives = ""

            options.stdout.write( "\t".join( ( 
                map_id2title[r.id],
                "%i" % r.start,
                "%i" % r.end,
                r.strand,
                r.arrangement,
                "%6.4f" % r.score,
                "%6.4f" % r.zscore,
                "%6.4e" % r.pvalue,
                "%6.4e" % r.qvalue,
                alternatives) ) )

            if options.add_sequence:
                s = masked_seqs[int(r.id)][r.start:r.end]
                if r.strand == "-": s = Genomics.complement( s )
                s = s[:6].upper() + s[6:-6].lower() + s[-6:].upper()
                options.stdout.write( "\t%s" % s )
                
            options.stdout.write("\n")
            noutput += 1

        # output stats
        if options.output_stats:
            outfile = E.openOutputFile( "fdr" )
            outfile.write("bin\thist\tnobserved\n" )
            for bin, hist, nobs in zip(matcher.bin_edges, matcher.hist, matcher.nobservations):
                outfile.write( "%f\t%f\t%f\n" % (bin, hist, nobs))
            outfile.close()


    elif options.fdr_control == "xall":

        matcher = Nubiscan.MatcherRandomisationSequence( sense_matrix,
                                                         samples = options.iterations )
    

        # collect all results
        matches = []
        for seq in FastaIterator.iterate(options.stdin):
            ninput += 1
            mm = matcher.run( seq.sequence,
                              options.arrangements,
                              qvalue_threshold = None )
            for m in mm:
                matches.append( m._replace( sequence = seq.title ) )

        # estimate qvalues for all matches across all sequences
        pvalues = [ x.pvalue for x in matches ]
        fdr = Stats.doFDR( pvalues )
        qvalues = fdr.mQValues
        results = []
        for m, qvalue in zip(matches, qvalues):
            if qvalue > options.qvalue_threshold: continue
            results.append( m._replace( qvalue = qvalue ) )

        if options.combine:            
            results =  Nubiscan.combineMotifs( results )

        # output
        for r in results:
            options.stdout.write( "\t".join( ( 
                r.id,
                "%i" % r.start,
                "%i" % r.end,
                r.strand,
                r.arrangement,
                "%6.4f" % r.score,
                "%6.4f" % r.zscore,
                "%6.4e" % r.pvalue,
                "%6.4e" % r.qvalue ) ) + "\n" )

            noutput += 1

    elif options.fdr_control == "per-sequence":
        matcher = Nubiscan.MatcherRandomisationSequence( sense_matrix,
                                                         samples = options.iterations )
    

        for seq in FastaIterator.iterate(options.stdin):
            ninput += 1
            result = matcher.run( seq.sequence,
                                  options.arrangements,
                                  qvalue_threshold = options.qvalue_threshold )
            
            if options.combine:
                result =  Nubiscan.combineMotifs( result )

            t = re.sub(" .*","",  seq.title)
            for r in result:
                options.stdout.write( "\t".join( ( 
                    t,
                    "%i" % r.start,
                    "%i" % r.end,
                    r.strand,
                    r.arrangement,
                    "%6.4f" % r.score,
                    "%6.4f" % r.zscore,
                    "%f" % r.pvalue,
                    "%f" % r.qvalue ) ) + "\n" )

            noutput += 1
    
    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()


        

if __name__ == "__main__":
    sys.exit( main( sys.argv) )






################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_make2help.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python cgat_make2help.py --help

Type::

   python cgat_make2help.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time

"""extract help test from a makefile
"""

import CGAT.Experiment as E
from CGAT.Local import getMakefiles, getScripts, getModules

class Parameter:
    def __init__(self, name, comment, default_value = "na"):
        self.mName = name
        self.mComment = comment
        self.mDefaultValue = value
        self.mValue = None        

    def printPretty( self ):
        lines = []

        if not self.mValue or self.mDefaultValue == self.mValue:
            value = "%s [default]" % self.mDefaultValue
        else:
            value = "%s [default=%s]" % (self.mValue, self.mDefaultValue )

        lines.append( "%s = %s" % (self.mName, value) )
        
        if self.mComment:
            lines.append( "" )
            lines.append( "  " + "\n  ".join( self.mComment ) )
        return "\n".join(lines)

class Target:
    def __init__(self, name, dependencies, comment, ):
        self.mName = name
        self.mComment = comment
        self.mDependencies = dependencies

    def printPretty( self ):
        lines = []
        
        lines.append( "%s" % self.mName )
        if self.mDependencies:
            lines.append( "" )
            lines.append( "  depends on: %s" % self.mDependencies )
        if self.mComment:
            lines.append( "" )
            lines.append( "  " + "\n  ".join( self.mComment ) )
        return "\n".join(lines)
            

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: cgat_make2help.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option( "-m", "--method", dest="method", type="choice",
                       help="method to use [t-test=t-test]",
                       choices=( "t-test", ) )
    parser.set_defaults(
        filename="Makefile",
        )
    
    (options, args) = E.Start( parser,
                                        quiet = True,
                                        add_pipe_options = True )

    src_dir = os.path.dirname( os.path.abspath( __file__  ) )

    makefiles = getMakefiles( (options.filename,), source_directory = src_dir, ignore_missing = True )

    ## variables starting with "PARAM"
    parameters = {}
    ## main targets: no patterns or no dependencies
    targets = []
    ## contain patterns and dependencies
    rules = []

    options.stdout.write( "Help for makefile %s\n\n" % os.path.abspath( options.filename ) )

    options.stdout.write( """This file is organized in three sections.

1. Primary Targets

   Primary targets are targets to be run by the users. Usually includes "all".

2. Parameters

   Parameters defined in the makefile.

3. Secondary targets and rules

   Rules defined in the makefile to build the primary targets.

""")

    for makefile in makefiles:
        
        infile = open(makefile, "r" )
        comment = []
        last_line = None
        
        first = True
        for line in infile:

            if last_line: 
                line = last_line + line
                last_line = None

            if not re.sub( "\s+", "", line): 
                first = False
                continue

            if line[-2] == "\\":
                last_line = line[:-2]
                continue
            
            if line[0] == "\t": continue

            if line[0] == "#":
                l = re.sub( "^#+\s*", "", line[:-1] )
                if l and not first:
                    comment.append( l )
                continue

            x = re.match("(PARAM_\S+[^?:])\s*\?*\s*=(.+)", line )
            if x:
                param,value = x.groups()
                if param not in parameters:
                    p = Parameter( param, comment, value )
                    parameters[param] = p 
                else:
                    p = parameters[param]
                    p.mComment += comment
                    p.mDefaultValue = value

            x = re.match("(PARAM_\S+[^?])\s*=(.+)", line )
            if x:
                param,value = x.groups()
                if param not in parameters:
                    p = Parameter( param, comment )
                    parameters[param] = p 
                else:
                    p = parameters[param]
                    p.mComment += comment
                    
                p.mValue = value

            x = re.match( "(\S+)\s*:\s*(.*)", line )
            if x:
                target, dependencies = x.groups()
                if "=" in target or "=" in dependencies: continue
                if target[0] in ".$": continue
                if "hook" in target: continue
                dependencies = re.sub( "\s+", " ", dependencies )
                if "." in target:
                    rules.append( Target( target, dependencies, comment ) )
                else:
                    targets.append( Target( target, dependencies, comment ) )

            comment = []
            first = False

        infile.close()

    options.stdout.write("\n\n Section 1: Primary targets\n" )
    options.stdout.write(" --------------------------\n\n" )
    options.stdout.write(""" Primary targets are targets to be run by the users. Usually includes "all".\n\n""")

    for x in targets:
        options.stdout.write( x.printPretty() + "\n\n" )

    options.stdout.write("\n\n Section 2: Parameters\n" )
    options.stdout.write(" --------------------------\n\n" )
    options.stdout.write(""" A list of parameters defined in the makefile.\n\n""")

    for x, y in sorted(parameters.items()):
        options.stdout.write( y.printPretty() + "\n\n\n" )        

    for x in targets:
        options.stdout.write( x.printPretty() + "\n\n" )

    options.stdout.write("\n\n Section 3: Secondary targets and rules\n" )
    options.stdout.write(" --------------------------\n\n" )
    options.stdout.write(""" Secondary targets are run by the pipeline.\n\n""")

    for x in rules:
        options.stdout.write( x.printPretty() + "\n\n" )
        
    options.stdout.write("\n\n Section 4: Contents\n" )
    options.stdout.write(" --------------------------\n\n" )
    options.stdout.write(" This help was created by scanning the contents of the following makefiles.\n" )

    for makefile in makefiles:
        options.stdout.write("    %s\n" % makefile )
        
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
maq2assembly.py - convert output from maq cns2view 
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python maq2assembly.py --help

Type::

   python maq2assembly.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse

USAGE="""python %s [OPTIONS] input1 input2

convert residue statistics (output from maq cns2view) to
a gff file.

Version: $Id: maq2assembly.py 2781 2009-09-10 11:33:14Z andreas $
""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Stats as Stats

def reader(infile):

    for line in infile:
        if line[0] == "#": continue
        if line.startswith("contig"): break

    last_contig, last_pos, start = None, 0, 0
    reads, qualities = [], []
    for line in infile:
        if line[0] == "#": continue
        contig, pos, qual, nreads, cov, bestqual = line[:-1].split("\t")
        pos, qual = int(pos)-1, int(qual)

        if contig != last_contig or pos - 1 != last_pos:
            if last_contig != None:
                yield last_contig, start, last_pos+1, reads, qualities
            start = pos
            last_contig = contig
            reads = []
            qualities = []

        last_pos = pos
        reads.append( nreads )
        qualities.append( qual )

    if last_contig != None:
        yield last_contig, start, last_pos+1, reads, qualities

class Builder:

    def __init__(self, genome_fasta, genome_queries, options ):

        self.mGenomeFasta = genome_fasta
        self.mQueriesFasta = queries_fasta
        self.options = options

        self.mIdFormat = options.output_format

        if self.options.output_filename_pattern:
            self.mOutFile = open( self.options.output_filename_pattern % self.mName, "w" )
        else:
            self.mOutFile = self.options.stdout

        self.mId = 0

    def __call__( self, id, *args ):

        self.mId = id

        self.mOutputId = self.mIdFormat % self.mId 
        self.process( *args )
        self.mOutFile.flush()

    def printHeader( self ):
        header = self.getHeader()
        if header:
            self.mOutFile.write( header + "\n" )

    def getHeader( self ):
        return None

class BuilderGFF(Builder):
    """output gff model of read
    """

    mName = "gff"

    def __init__(self, *args, **kwargs ):
        Builder.__init__( self, *args, **kwargs )
       
    def getHeader( self ):
        return None

    def process( self, contig, start, end, reads, qualities ):

        entry = GTF.Entry()
        entry.start, entry.end = start, end
        entry.gene_id = self.mIdFormat % id
        entry.transcript_id = entry.gene_id
        entry.contig = contig
        entry.feature = "exon"
        entry.source = "maq"

        read_stats = Stats.Summary( reads )

        entry.score = "%5.2f" % read_stats['mean']

        self.mOutFile.write( str(entry) + "\n" )

class BuilderCoverage(Builder):
    """compute residue read coverage over all genomic bases that have at least one aligned match
    """

    mName = "coverage"

    def __init__(self, *args, **kwargs ):
        Builder.__init__( self, *args, **kwargs )
       
    def getHeader( self ):
        return "id\tcontig\tstart\tend\tsize\tnmatches\tncovered\t%s" % ("\t".join(Stats.DistributionalParameters().getHeaders()) )

    def process( self, contig, start, end, reads, qualities ):

        aligned = filter( lambda x: x > 0, reads)
        self.mOutFile.write( "%s\t%s\t%i\t%i\t%i\t%i\t%i\t%s\n" % (self.mOutputId, 
                                                                   contig, start, end, end - start, 
                                                                   len(reads),
                                                                   len(aligned),
                                                                   str(Stats.DistributionalParameters( aligned ) )))

class BuilderQuality(Builder):
    """compute residue quality over all genomic bases that have at least one aligned match
    """

    mName = "quality"

    def __init__(self, *args, **kwargs ):
        Builder.__init__( self, *args, **kwargs )
       
    def getHeader( self ):
        return "id\tcontig\tstart\tend\tsize\tnmatches\tncovered\t%s" % ("\t".join(Stats.DistributionalParameters().getHeaders()) )

    def process( self, contig, start, end, reads, qualities ):

        self.mOutFile.write( "%s\t%s\t%i\t%i\t%i\t%i\t%i\t%s\n" % (self.mOutputId, 
                                                                   contig, start, end, end - start, 
                                                                   len(reads),
                                                                   len(qualities),
                                                                   str(Stats.DistributionalParameters( qualities ) )))

class BuilderRegion(Builder):

    mName = "regions"

    def __init__(self, *args, **kwargs ):
        Builder.__init__( self, *args, **kwargs )

    def getHeader( self ):
        return "id\tcontig\tstart\tend\tsize\tnmatches"

    def process( self, contig, start, end, reads, qualities ):
        self.mOutFile.write( "%s\t%s\t%i\t%i\t%i\t%i\n" % (self.mOutputId, contig, start, end, end - start, len(reads) ))
        self.mOutFile.flush()

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: maq2assembly.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option( "-f", "--forward-coordinates", dest="forward_coordinates", 
                      help="translate to forward coordinates.", action="store_true"  )

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string" ,
                       help="OUTPUT filename pattern for additional data [%default].")

    parser.add_option( "--method", dest="methods", type="choice", action="append",
                       choices=( "gff", "coverage", "region", "quality" ),
                       help="methods to apply [%default].")

    parser.set_defaults(
        output_format = "%08i",
        output_filename_pattern = "%s",
        methods = [],
        )

    (options, args) = E.Start( parser )

    ################################################
    ################################################
    ################################################
    ## pick a processor
    ################################################        
    methods = []

    if len(options.methods) == 0:
        raise "please supply at least one method to apply."

    genome_fasta, queries_fasta = None, None

    for method in options.methods:
        if method == "gff":
            methods.append( BuilderGFF(  genome_fasta, queries_fasta, options) )
        elif method == "coverage":
            methods.append( BuilderCoverage( genome_fasta, queries_fasta, options) )
        elif method == "quality":
            methods.append( BuilderQuality( genome_fasta, queries_fasta, options) )
        elif method == "region":
            methods.append( BuilderRegion( genome_fasta, queries_fasta, options) )

    for method in methods:
        method.printHeader()

    ninput, noutput = 0, 0
    id = 0
    for contig, start, end, reads, qualities in reader(options.stdin):

        ninput += 1
        id += 1
        for m in methods:
            m( id, contig, start, end, reads, qualities )

        noutput += 1

    options.stdlog.write("# ninput=%i, noutput=%i\n" % (ninput, noutput) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: fasta2bed.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
fasta2bed.py - segment sequences
=====================================================

:Author: Andreas Heger
:Release: $Id: fasta2bed.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

This script takes a genomic sequence in :term:`fasta` format
and applies various segmentation algorithms.

The methods implemented (``--methods``) are:

GCProfile
   use `GCProfile <http://tubic.tju.edu.cn/GC-Profile/>`_ to segment by
   G+C content. GCProfile needs to be installed and in the PATH.

cpg
   output all locations of cpg in the genome

fixed-width-windows-gc
   output fixed width windows of a certain size adding their
   G+C content as score

gaps
   ouput all locations of assembly gaps (blocks of `N`)
   in the genomic sequences

Usage
-----

Type::

   python fasta2bed.py --method=gap < in.fasta > out.bed


Type::

   python fasta2bed.py --help

for command line help.

Command line options
--------------------

""" 

import os
import sys
import re
import optparse
import tempfile
import subprocess
import glob
import shutil

import CGAT.Experiment as E

EXECUTABLE="GCProfile"

import CGAT.FastaIterator as FastaIterator

def segmentWithCpG( infile, options ):
    '''segment a fasta file, output locations of CpG.'''

    ninput, nskipped, noutput = 0, 0, 0
    
    iterator = FastaIterator.FastaIterator( infile )

    segments = []
    
    while 1:
        try:
            cur_record = iterator.next()
        except StopIteration:
            break

        if cur_record is None: break
        ninput += 1
        contig = re.sub("\s.*", "", cur_record.title )
        last = None
        for pos, this in enumerate( cur_record.sequence.upper()):
            if last == "C" and this == "G":
                segments.append( (contig, pos - 1, pos + 1, 1.0))
            last = this

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    return segments

def segmentWithGCProfile( infile, options ):
    '''segment a fasta file with GCProfile.'''

    ninput, nskipped, noutput = 0, 0, 0
    
    iterator = FastaIterator.FastaIterator( infile )

    tmpdir = tempfile.mkdtemp()
    E.info( "working in %s" % tmpdir )
    
    while 1:
        try:
            cur_record = iterator.next()
        except StopIteration:
            break

        if cur_record is None: break
        ninput += 1
        
        contig = re.sub("\s.*", "", cur_record.title )
        filename = os.path.join( tmpdir, contig) + ".fasta" 
        with open( filename, "w") as f:
            f.write( ">%s\n%s\n" % (contig, cur_record.sequence ) )
        
        E.info( "running %s on %s" % (EXECUTABLE, contig ) )

        statement = "%s %s -t %i -g %f -i %i " % \
                    (EXECUTABLE, filename,
                     options.gcprofile_halting_parameter,
                     options.gcprofile_gap_size,
                     options.gcprofile_min_length )
        
        E.debug( "statement = %s" % statement )

        process = subprocess.Popen(  statement,
                                     cwd = tmpdir,
                                     shell = True,
                                     stdin = None,
                                     stdout = subprocess.PIPE,
                                     stderr = subprocess.PIPE )


        stdout, stderr = process.communicate()

        E.debug( "GProfile: stdout=%s" % stdout )
        E.debug( "GProfile: stderr=%s" % stderr )
                
        # process.stdin.close()
        if process.returncode != 0:
            raise OSError( "Child was terminated by signal %i: \nThe stderr was: \n%s\n%s\n" % (-process.returncode, stderr, statement ))

        noutput += 1

    segments = []

    for f in glob.glob( os.path.join( tmpdir, "*.SegGC" ) ):
        contig = os.path.basename(f[:-len(".SegGC")])
        with open(f,"r") as infile:
            start = None
            for line in infile:
                pos, gc = line[:-1].split("\t")
                try: gc = float(gc)
                except ValueError: gc = None
                end = int(pos)
                # ignore the double lines
                if gc != None and start != None and end - start > 2:
                    segments.append( (contig, start, end, gc ) )
                start = int(pos)-1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    # shutil.rmtree( tmpdir )
    
    return segments

def segmentFixedWidthWindows( infile, window_size ):
    """return a list of fixed contig sizes."""

    ninput, nskipped, noutput = 0, 0, 0
    
    iterator = FastaIterator.FastaIterator( infile )
    window_increment = window_size
    # at most 50% can be gap
    gap_cutoff = int( window_size // 2 )
    segments = []
            
    while 1:
        ninput += 1
        try:
            cur_record = iterator.next()
        except StopIteration:
            break

        if cur_record is None: break
        contig = re.sub("\s.*", "", cur_record.title )
        seq = cur_record.sequence
        size = len(cur_record.sequence)
        
        for x in range(0, size, window_increment):
            s = seq[x:x+window_size].upper()
            gc, at = 0,0
            for c in s:
                if c in "GC": gc += 1
                elif c in "AT": at += 1

            # skip segments containing mostly gaps
            if window_size - (gc + at) > gap_cutoff:
                nskipped += 1
                continue
            
            segments.append( (contig, x, x+window_size, float(gc) / (gc+at) ) )
        noutput += 1
        
    E.info( "ninput=%i, noutput=%i, nskipped_windows=%i" % (ninput, noutput,nskipped) )

    return segments

def segmentGaps( infile, gap_char ):

    iterator = FastaIterator.FastaIterator( infile )

    while 1:
        try:
            cur_record = iterator.next()
        except StopIteration:
            break

        if cur_record is None: break
        contig = re.sub("\s.*", "", cur_record.title )

        if cur_record.sequence[0] in gap_char:
            first_res = 0
        else:
            first_res = -1
        for x, c in enumerate( cur_record.sequence ):
            if c not in gap_char:
                if first_res >= 0:
                    yield ( contig, first_res, x, 0 )
                first_res = -1
            else:
                if first_res < 0:
                    first_res = x

            x += 1
            
        if first_res > 0:
            yield( contig, first_res, x, 0 )
            
def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: fasta2bed.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--gap-size", dest="gap_size", type="float",
                      help="GCProfile: Choose n as the gap size to be filtered "
                      "(If n > 1, n bp is set as the gap size to be filtered. "
                      "If 0 < n < 1, for example, n = 0.01, means gaps less than 1% "
                      "of the input sequence length will be filtered [default=%default]."  )

    parser.add_option("-i", "--min-length", dest="gcprofile_min_length", type="int",
                      help="GCProfile: minimum length [default=%default]" )

    parser.add_option( "--halting-parameter", dest="gcprofile_halting_parameter", type="int",
                       help="GCProfile: halting parameter [default=%default]" )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices = ("GCProfile", 
                                 "fixed-width-windows-gc",
                                 "cpg",
                                 "gaps",),
                      help="Method to use for segmentation [default=%default]" )

    parser.add_option( "-w", "--window-size=", dest="window_size", type="int",
                       help="window size for fixed-width windows [default=%default]." )
    
    parser.set_defaults(
        gcprofile_halting_parameter = 1000,
        gcprofile_gap_size = 0.01,
        gcprofile_min_length = 3000,
        window_size = 10000,
        method = "GCProfile",
        gap_char = "NnXx"
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth

    if options.method == "GCProfile":
        segments = segmentWithGCProfile( options.stdin, options )
    elif options.method == "cpg":
        segments = segmentWithCpG( options.stdin, options )
    elif options.method == "Isoplotter":
        segments = segmentWithIsoplotter( options.stdin, options )
    elif options.method == "fixed-width-windows-gc":
        segments = segmentFixedWidthWindows( options.stdin, options.window_size )
    elif options.method == "gaps":
        segments = segmentGaps( options.stdin, options.gap_char )
    else:
        raise ValueError("unknown method %s" % (method))
    x = 0
    for contig, start, end, gc in segments:
        x += 1
        options.stdout.write( "%s\n" % "\t".join( \
            (contig, str(start), str(end), str(x), "%6.4f" % (100.0 * gc) ) ) )
        
    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2annotator2tsv.py - convert from gff to annotator format
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python gff2annotator2tsv.py --help

Type::

   python gff2annotator2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse
import time
import os
import shutil
import tempfile
import math
import itertools
import glob
import collections

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.IndexedFasta as IndexedFasta
import CGAT.IOTools as IOTools
import CGATPipelines.PipelineEnrichment as PipelineEnrichment 

USAGE="""python %s [OPTIONS] < stdin > stdout

convert a gff file into annotator compatible regions. Depending on the option --section
this script will create:

   segments
      a segments file

   annotations
      a file with annotations. Multiple gff files can be provided. 

   annotations-genes
      if gtf files are provided (ending in .gtf), multiple subsets can be created using the 
      --subsets options. In this case a list of gene_ids for each subset is required.

   annotations-go 
      a file with annotations. Input is a gtf file and a map of gene identifiers
      to categories. Requires --input-filename-map.

   annotations-gff
      take all annotations in a gff file and create individual annotations for each feature 
      encountered. Multiple files will be aggregated if they contain the same feature.

   workspace
      a file with a workspace

""" % sys.argv[0]

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2annotator2tsv.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"])

        
    parser.add_option( "-g", "--genome-file", dest="genome_file", type="string",
                       help="filename with genome."  )

    parser.add_option( "-f", "--features", dest="features", type="string", 
                       help="feature to collect [default=None]."  )

    parser.add_option( "-i", "--files", dest="files", action="append",
                       help="use multiple annotations [default=None]."  )

    parser.add_option(  "-a", "--annotations", dest="annotations", type="string", 
                       help="aggregate name for annotations if only single file is provided from STDIN [default=None]."  )

    parser.add_option(  "--input-filename-map", dest="input_filename_map", type="string", 
                       help="filename with a map of gene_ids to categories [default=None]."  )

    parser.add_option(  "--output-filename-synonyms", dest="output_filename_synonyms", type="string", 
                       help="output filename for synonyms. For workspace building, the gff source will be used as the id (instead of the contig) [default=None]."  )

    parser.add_option( "-m", "--max-length", dest="max_length", type="string", 
                       help="maximum segment length [default=None]."  )

    parser.add_option( "-s", "--section", dest="section", type="choice", 
                       choices=("segments", "annotations", "annotations-genes", "annotations-go", "workspace", "annotations-gff" ),
                       help="annotator section [default=None]."  )

    parser.add_option( "--subset", dest="subsets", type="string", action="append",
                       help="add filenames to delimit subsets within the gff files. The syntax is filename.gff,label,filename.ids [default=None]."  )

    parser.add_option( "--remove-regex", dest="remove_regex", type="string", 
                       help="regular expression of contigs to remove [default=None]."  )

    parser.set_defaults(
        genome_file = None,
        feature = None,
        section = "segments",
        annotations = "annotations",
        max_length = 100000,
        files = [],
        subsets = [],
        input_filename_map = None,
        output_filename_synonyms = None,
        input_format = "gff",
        remove_regex = None,
        )

    (options, args) = E.Start( parser )

    options.files += args
    if len(options.files) == 0: options.files.append("-")
    options.files = list( itertools.chain( *[ re.split( "[,; ]+", x) for x in options.files ] ) )

    if options.subsets:
        subsets = collections.defaultdict( list )
        for s in options.subsets: 
            filename_gff,label,filename_ids = s.split( "," )
            subsets[filename_gff].append( (label,filename_ids) )
        options.subsets = subsets

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        fasta = None

    if options.section == "segments":
        prefix = "##Segs"
    elif options.section.startswith( "annotations" ):
        prefix = "##Id"
    elif options.section == "workspace":
        prefix = "##Work"
    else:
        raise ValueError("unknown section %s" % options.section)
        
    ninput, ncontigs, nsegments, ndiscarded = 0, 0, 0, 0

    if options.remove_regex:
        options.remove_regex = re.compile( options.remove_regex )

    if options.section in ("segments", "workspace"):

        iterator = GTF.iterator_filtered( GFF.iterator( options.stdin ),
                                          feature=options.feature )

        if options.output_filename_synonyms:
            outfile_synonyms = open(options.output_filename_synonyms, "w")
            with_records = True
        else:
            outfile_synonyms = None
            with_records = False

        intervals =GTF.readAsIntervals( iterator, with_records = with_records )
        ninput, nsegments, ndiscarded, ncontigs = \
            PipelineEnrichment.outputSegments( options.stdout,
                                               intervals,
                                               options.section,
                                               outfile_synonyms = outfile_synonyms,
                                               max_length = options.max_length,
                                               remove_regex = options.remove_regex )
            
        if outfile_synonyms:
            outfile_synonyms.close()

    elif options.section == "annotations-go":

        assert options.input_filename_map, "please supply option --input-filename-map" 

        iterator = GTF.iterator_filtered( GTF.iterator( options.stdin ),
                                          feature=options.feature )

        geneid2categories = IOTools.readMultiMap( open( options.input_filename_map, "r") )

        category2segments = collections.defaultdict( list )

        for contig, gffs in GTF.readAsIntervals( iterator, with_gene_id = True ).items():
            if options.remove_regex and options.remove_regex.search( contig ): continue
            
            for start, end, geneid in gffs:
                if geneid not in geneid2categories: continue
                for category in geneid2categories[geneid]:
                    category2segments[category].append(nsegments)

                options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, start, end ) )
                nsegments += 1                        
            
        for category, segments in category2segments.iteritems():
            options.stdout.write("##Ann\t%s\t%s\n" % (category, "\t".join( ["%i" % x for x in segments ] ) ) )
            E.info( "set %s annotated with %i segments" % (category, len(segments)) )

    elif options.section == "annotations":

        for filename in options.files:

            E.info( "adding filename %s" % filename )

            start = nsegments
            is_gtf = False

            if filename == "-":
                iterator = GTF.iterator_filtered( GFF.iterator( sys.stdin ),
                                                  feature=options.feature )
                filename = options.annotations
            elif filename.endswith(".gtf"):
                is_gtf = True
                with open( filename, "r") as infile:
                    iterator = GTF.iterator_filtered( GTF.iterator( infile ),
                                                      feature=options.feature )
                
            else:
                with open( filename, "r") as infile:
                    iterator = GTF.iterator_filtered( GFF.iterator( infile ),
                                                      feature=options.feature )
           
            E.debug("processing %s" % (filename))

            if not options.subsets or filename not in options.subsets:
                for contig, gffs in GTF.readAsIntervals( iterator ).items():
                    if options.remove_regex and options.remove_regex.search( contig ): continue

                    for x in gffs:
                        options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, x[0], x[1] ) )
                        nsegments += 1

                options.stdout.write("##Ann\t%s\t%s\n" % (filename, "\t".join( ["%i" % x for x in range(start, nsegments) ] ) ) )
                E.info( "set %s annotated with %i segments" % (filename, nsegments - start) )

            else:
                raise ValueError("don't know how to filter %s" % filename )

    elif options.section == "annotations-gff":

        for filename in options.files:
            if filename == "-":
                iterator = GTF.iterator( sys.stdin )
            else:
                iterator = GTF.iterator_filtered( GFF.iterator( open( filename, "r") ) )

            segments = collections.defaultdict( list )
            for gff in iterator:
                segments[":".join((gff.source,gff.feature))].append( (gff.contig,gff.start, gff.end) )
  
            feature2segments = {}

            for feature, s in segments.iteritems():
                s.sort()

                s1 = nsegments

                for contig, start, end in s:
                    if options.remove_regex and options.remove_regex.search( contig ): continue

                    options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, start, end ) )
                    nsegments += 1

                feature2segments[feature] = (s1, nsegments)
            
        for feature, id_range in feature2segments.iteritems():
            start, end = id_range
            options.stdout.write("##Ann\t%s\t%s\n" % (feature, "\t".join( ["%i" % x for x in xrange( start,end) ] ) ) )
            E.info( "set %s annotated with %i segments" % (feature, end-start) )

    elif options.section == "annotations-genes":

        for filename in options.files:

            E.info( "adding filename %s" % filename )

            start = nsegments

            assert filename.endswith(".gtf") or filename.endswith(".gtf.gz"), \
                "requiring .gtf files for gene list filtering, received %s" % filename

            infile = IOTools.openFile( filename )
            iterator = GTF.iterator_filtered( GTF.iterator( infile ),
                                              feature=options.feature )
                
            E.debug("processing %s" % (filename))
            
            if not options.subsets or filename not in options.subsets:
                ## output all
                for contig, gffs in GTF.readAsIntervals( iterator ).items():
                    if options.remove_regex and options.remove_regex.search( contig ): continue

                    for x in gffs:
                        options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, x[0],x[1] ) )
                        nsegments += 1

                options.stdout.write("##Ann\t%s\t%s\n" % (filename, "\t".join( ["%i" % x for x in range(start, nsegments) ] ) ) )
                E.info( "set %s annotated with %i segments" % (filename, nsegments - start) )

            else:
                ## create subsets
                E.debug("applying subsets for %s" % filename )
                geneid2label, label2segments = collections.defaultdict(list) , {}
                for label, filename_ids in options.subsets[filename]:
                    gene_ids = IOTools.readList( open(filename_ids, "r") )
                    for gene_id in gene_ids: geneid2label[gene_id].append( label )
                    label2segments[label] = []

                for contig, gffs in GTF.readAsIntervals( iterator, with_gene_id = True ).items():

                    if options.remove_regex and options.remove_regex.search( contig ): continue

                    for start, end, gene_id in gffs:
                        if gene_id not in geneid2label: continue
                        for label in geneid2label[gene_id]:
                            label2segments[label].append(nsegments)
                            
                        options.stdout.write( "%s\t%i\t%s\t(%i,%i)\n" % (prefix, nsegments, contig, start, end ) )
                        nsegments += 1                        
                        
                for label, segments in label2segments.iteritems():
                    options.stdout.write("##Ann\t%s\t%s\n" % (label, "\t".join( ["%i" % x for x in segments ] ) ) )
                    E.info( "set %s (%s) annotated with %i segments" % (label, filename, len(segments)) )

    E.info( "ninput=%i, ncontigs=%i, nsegments=%i, ndiscarded=%i" % (ninput, ncontigs, nsegments, ndiscarded))

    E.Stop()






################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
histograms2distance.py - compute Kullback-Leibler distance
==========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Output the Kullback-Leibler distance between two or
more distributions given as histograms.

Usage
-----

Example::

   python histograms2kl.py --help

Type::

   python histograms2kl.py --help

for command line help.

This script was formerly called :file:`compare_histograms.py`.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time
import random

import math
import numpy
import CGAT.Experiment as E
import pgdb
import CGAT.IOTools as IOTools

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: compare_histograms.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-m", "--method", dest="method", type="choice",
                       help="method to use [kl=kullback-leibler]",
                       choices=("kl",) )
    parser.add_option( "-n", "--no-normalize", dest="normalize", action="store_false",
                       help="do not normalize data" )
    parser.add_option( "-p", "--pseudocounts", dest="pseudocounts", type="int",
                       help="pseudocounts to add." )
    parser.add_option( "-f", "--number-format", dest="number_format", type="string",
                       help="number format." )

    parser.set_defaults(
        method = "kl",
        columns = "all",
        headers = True,
        xrange = None,
        pseudocounts = 1,
        normalize = True,
        number_format = "%6.4f"
        )
    
    (options, args) = E.Start( parser,
                                        add_pipe_options = True )

    if options.xrange: options.xrange = map(float, options.xrange.split(","))

    data, legend = IOTools.readTable( sys.stdin,
                                      numeric_type=numpy.float32,
                                      take=options.columns,
                                      headers = options.headers,
                                      truncate= options.xrange )

    nrows, ncols = data.shape

    ## first: normalize rows
    for y in range(1, ncols):
        for x in range(nrows): data[x,y] = data[x,y] + float(options.pseudocounts)
        if options.normalize:
            t = numpy.sum( data[:,y] )            
            for x in range(nrows): data[x, y] = data[x,y] / t

    for x in range(1, len(legend) - 1):
        for y in range(x+1, len(legend)):

            if options.method == "kl":
                d1 = 0.0
                d2 = 0.0
                for bin in range(nrows):
                    p = data[bin,x]
                    q = data[bin,y]
                    d1 += p * math.log( p / q)
                    d2 += q * math.log( q / p)
                    
                options.stdout.write( "%s\t%s\t%s\n" %
                                      (legend[x], legend[y],
                                       options.number_format % d1) )
                options.stdout.write( "%s\t%s\t%s\n" %
                                      (legend[y], legend[x],
                                       options.number_format % d2) )
                            
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
GO.py - compute GO enrichment from gene lists
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, getopt, time, optparse, math, tempfile, subprocess, random
import collections

import scipy
import scipy.stats
import scipy.special
import numpy
import Stats as Stats
import Database as Database
import Experiment as E
import IOTools as IOTools
import CSV as CSV

from rpy2.robjects import r as R

MIN_FLOAT = sys.float_info.min
# The following code was taken from:
#
# http://mail.python.org/pipermail/python-list/2006-January/359797.html
#
#
def lnchoose(n, m):
    nf = scipy.special.gammaln(n + 1)
    mf = scipy.special.gammaln(m + 1)
    nmmnf = scipy.special.gammaln(n - m + 1)
    return nf - (mf + nmmnf)

def hypergeometric_gamma(k, n1, n2, t):
    if t > n1 + n2:
        t = n1 + n2
    if k > n1 or k > t:
        return 0
    elif t > n2 and ((k + n2) < t):
        return 0
    else:
        c1 = lnchoose(n1,k)
        c2 = lnchoose(n2, t - k)
        c3 = lnchoose(n1 + n2 ,t)

    # print "hyperg:", k, n1, n2, t, math.exp(c1 + c2 - c3)
    return max( math.exp(c1 + c2 - c3), MIN_FLOAT )

def hypergeometric_P( k, n0, n1, t):

    GSL_DBL_EPSILON=1e-10

    assert t <= (n0+n1), "t larger than population size"
    assert n0 >= 0, "n0 < 0"
    assert n1 >= 0, "n1 < 0"

    if k >= n0 or k >= t:
        P = 1.0
    elif (k < 0.0):
        P = 0.0
    else:
        P = 0.0;
        mode = int( float(t*n0) / float(n0+n1))
        relerr = 1.0;
        if k < mode :
            i = k;
            relerr = 1.0;
            while(i >= 0 and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i -= 1
        else:
            i = mode
            relerr = 1.0;
            while(i <= k and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i += 1
            i = mode - 1;
            relerr = 1.0;
            while( i >=0 and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i -= 1
    return P

def hypergeometric_Q( k, n0, n1, t):

    GSL_DBL_EPSILON=1e-10
    assert t <= (n0+n1), "t larger than population size"    
    assert n0 >= 0, "n0 < 0"
    assert n1 >= 0, "n1 < 0"
    if k >= n0 or k >= t:
        P = 1.0
    elif (k < 0.0):
        P = 0.0
    else:
        P = 0.0;
        mode = int(float(t * n0) / float(n0+n1))
        relerr = 1.0
        if k < mode :
            i = mode
            relerr = 1.0
            while(i <= t and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i += 1
            i = mode - 1
            relerr = 1.0;
            while( i > k and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i -= 1
            
        else:
            i = k + 1
            relerr = 1.0
            while(i <= t and relerr > GSL_DBL_EPSILON and P < 1.0):
                tmp = hypergeometric_gamma( i, n0, n1, t)
                P += tmp
                relerr = tmp / P
                i += 1
    return P


class Error(Exception):
    """Base class for exceptions in this module."""
    def __str__(self):
        return str(self.message)
    def _get_message(self, message): return self._message
    def _set_message(self, message): self._message = message
    message = property(_get_message, _set_message)

class InputError(Error):
    """Exception raised for errors in the input.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class GOEntry:
    
    mNameSpaceMap= {
        'molecular_function' : 'mol_function',
        'cellular_component' : 'cell_location',
        'biological_process' : 'biol_process', 
        }
    
    def __init__(self, default_namespace = "ontology" ):
        self.mNameSpace = default_namespace

    def fromOBO( self, section ):
        """read entry form an OBO formatted file."""

        self.mIsA = []

        for line in section:
            
            data = line[:-1].split(":")
            term = data[0]
            rest = ":".join( data[1:] ).strip()
            if term == "name": self.mName = rest
            elif term == "id": self.mId = rest
            elif term == "namespace": self.mNameSpace = self.mNameSpaceMap.get(rest, rest)
            elif term == "def": self.mDefinition = rest
            elif term == "exact_synonym": self.mSynonym = rest
            elif term == "is_a": self.mIsA.append( rest )
            elif term == "comment": self.mComment = rest
            elif term == "is_obsolete": self.mIsObsolete = True
            
##-------------------------------------------------------------------------------

def readOntology( infile ):
    """read ontology in OBO format from infile.

    returns a dictionary of Ontology entries.
    """
    result = {}

    def iterate_blocks( infile ):

        lines = []
        
        for line in infile:
            if line.strip() == "": 
                if lines: yield lines
                lines = []
                continue

            lines.append( line )

    default_namespace = "ontology"

    for section in iterate_blocks( infile ):

        if section[0].startswith( "[Term]"):
            go = GOEntry( default_namespace = default_namespace )
            go.fromOBO( section )
            result[go.mId] = go
        else:
            for line in section:
                data = line[:-1].split(":")
                if data[0] ==  "default-namespace":
                    default_namespace = data[1].strip()
            
    return result

##-------------------------------------------------------------------------------
class GOSample:
    """store results from sampling.
    """
    def __init__(self, mmin, mmax, mmean, mstddev, mprobovers, mprobunders, counts):

        self.mMin = mmin
        self.mMax = mmax
        self.mMean = mmean
        self.mStddev = mstddev
        self.mProbabilitiesOverRepresentation = mprobovers
        self.mProbabilitiesUnderRepresentation = mprobunders
        self.mCounts = counts

##-------------------------------------------------------------------------------
class GOResult:

    mIsOverRepresented = False
    mGOId = None
    mSampleCountsCategory = 0
    mBackgroundCountsCategory = 0
    mSampleCountsTotal = 0
    mBackgroundCountsTotal = 0
    mProbabilityOverRepresentation = 0
    mProbabilityUnderRepresentation = 0
    mPValue = 1.0

    def __init__(self, goid = None):
        self.mGOId = goid

    def UpdateProbabilities( self ):
        """calculate probabilities for given counts.

        """
        if self.mBackgroundCountsTotal == 0:
            return

        # various sanity checs
        assert self.mBackgroundCountsCategory >= self.mSampleCountsCategory, \
            "%s: more counts in foreground (%i) than in the background (%i) - make sure the foreground is part of the background." %\
            (self.mGOId, self.mSampleCountsCategory, self.mBackgroundCountsCategory )

        assert self.mBackgroundCountsTotal >= self.mBackgroundCountsCategory, \
            "%s: background: more counts in category (%i) than in total (%i)." %\
            (self.mGOId, self.mBackgroundCountsCategory, self.mBackgroundCountsTotal)

        assert self.mSampleCountsTotal >= self.mSampleCountsCategory, \
            "%s: forerground: more counts in category (%i) than in total (%i)." %\
            (self.mGOId, self.mSampleCountsCategory, self.mSampleCountsTotal)

        if self.mSampleCountsCategory == 0:
            self.mProbabilityOverRepresentation = 1.0
        else:
            self.mProbabilityOverRepresentation = hypergeometric_Q( self.mSampleCountsCategory - 1,
                                                                    self.mBackgroundCountsCategory,
                                                                    self.mBackgroundCountsTotal - self.mBackgroundCountsCategory,
                                                                    self.mSampleCountsTotal )
        
        self.mProbabilityUnderRepresentation = hypergeometric_P( self.mSampleCountsCategory,
                                                                 self.mBackgroundCountsCategory,
                                                                 self.mBackgroundCountsTotal - self.mBackgroundCountsCategory,
                                                                 self.mSampleCountsTotal )
        
        self.mPValue = min(self.mProbabilityOverRepresentation, self.mProbabilityUnderRepresentation) 

        if self.mSampleCountsTotal == 0 or self.mBackgroundCountsCategory == 0:
            self.mRatio = "na"
        else:
            self.mRatio = float(self.mSampleCountsCategory) * self.mBackgroundCountsTotal / self.mSampleCountsTotal / self.mBackgroundCountsCategory 

    def getHeaders( self ):
        return ["scount", "stotal", "spercent", 
                "bcount", "btotal", "bpercent", 
                "ratio",
                "pvalue", "pover", "punder" ]

    def __str__(self):
        """return string representation."""        
        return "%i\t%i\t%s\t%i\t%i\t%s\t%s\t%6.4e\t%6.4e\t%6.4e" % (self.mSampleCountsCategory,
                                                                    self.mSampleCountsTotal,
                                                                    IOTools.prettyPercent( self.mSampleCountsCategory, self.mSampleCountsTotal ),
                                                                    self.mBackgroundCountsCategory,
                                                                    self.mBackgroundCountsTotal,
                                                                    IOTools.prettyPercent( self.mBackgroundCountsCategory, self.mBackgroundCountsTotal ),
                                                                    IOTools.prettyFloat( self.mRatio ),
                                                                    self.mPValue,
                                                                    self.mProbabilityOverRepresentation,
                                                                    self.mProbabilityUnderRepresentation )

##-------------------------------------------------------------------------------                                           
class GOResults:
    '''container for go results.'''
    def __init__(self):
        # dictionary of (GOID,GoResult) tuples
        self.mResults = {}
        self.mNumGenes = 0
        self.mBackgroundCountsTotal = 0
        self.mSampleCountsTotal = 0

    def __str__(self):
        """return string representation."""
        lines = []
        lines.append( "\t".join(map(str, (self.mNumGenes, self.mBackgroundCountsTotal, self.mSampleCountsTotal))))
        for k, v in self.mResults.items():
            lines.append("%s\t%s" % (k, str(v)))
        return "\n".join(lines)

##-------------------------------------------------------------------------------
class GOInfo:
    mGOId = None
    mGOType = None
    mDescription = None
    
    def __init__( self,
                  goid = None,
                  go_type = None,
                  description = None ):

        self.mDescription = description
        self.mGOId = goid
        self.mGOType = go_type

    def __str__(self):
        if self.mGOId == None:
            return "\t".join(map(str, ("", "", "")))
        else:
            return "\t".join(map(str, (self.mGOId, self.mGOType, self.mDescription)))
    
    def getHeaders( self ):
        return ["goid", "go_catagory", "go_description" ]

##-------------------------------------------------------------------------------
class GOMatch(GOInfo):
    mEvidence = None
    
    def __init__( self,
                  goid = None,
                  go_type = None,
                  description = None,
                  evidence = None ):

        GOInfo.__init__(self, goid, go_type, description)
        self.mEvidence = evidence

    def __str__(self):
        return "\t".join(map(str, (self.mGOId, self.mGOType, self.mDescription, self.mEvidence)))

##---------------------------------------------------------------------
def FilterByGOIds(  gene2go, go2info ):

    """
    filter gene_id to go_id lookup by a list of go_ids

    returns a new gene2go mapping.
    
    used to restrict GO terms to GO_slim and remove alternates

    gene2go 				# starting set, map of genes to go terms
    go2info				# alt ids are repeats of superceding ids
    """

    filtered_gene2go = {}

    for gene_id in gene2go.keys():
        new_go = set()
        for go in gene2go[gene_id]:
            if go.mGOId in go2info:
                new_go.add( go )

        if new_go:
            filtered_gene2go[gene_id] = list(new_go)

    return filtered_gene2go

##---------------------------------------------------------------------
def MapGO2Slims(  gene2go, go2slim, ontology = None ):
    """filter gene2go lookup by a list of go_ids in go2slim.

    gene2go: map of genes to go terms
    go2slim: map of go categories to goslim go categories   

    If ontology is given, missing descriptions of go entries
    are added from the ontology.

    returns a new gene2go mapping.
    """

    ## build map of go identifiers to go info
    map_go2info = {}
    if ontology:
        for go in ontology.values():
            map_go2info[go.mId] = GOInfo( goid=go.mId, 
                                          go_type = go.mNameSpace,
                                          description = go.mName )
    else:
        for gene_id, gos in gene2go.items():
            for go in gos:
                map_go2info[go.mGOId] = go

    filtered_gene2go = {}

    for gene_id, gos in gene2go.items():    
        new_go = set()
        for go in gos:
            if go.mGOId in go2slim:
                for gg in go2slim[go.mGOId]:
                    if gg in map_go2info:
                        new_go.add( map_go2info[gg] )
                    else:
                        raise IndexError, "description for mapped go term not present: %s -> %s" % (go.mGOId, gg)
        if new_go:
            filtered_gene2go[gene_id] = list(new_go)

    return filtered_gene2go

##------------------------------------------------------------------------------
def GetGOSlims( infile ):
    """
    returns a map of go identifiers to slim categories

    Input is the output of Chris Mungal's map2slim.pl.
    """

    go2go = {}
    for line in infile:
        if line[:len("part_of")] == "part_of": continue

        mapped, parents = line.split("//")
        go, goslims = mapped.split("=>")
        goslims = goslims.split(" ")
        if len(goslims) == 0 : continue

        go2go[go.strip()] = filter( lambda x: len(x), map(lambda x: x.strip(), goslims))
        
    return go2go

##------------------------------------------------------------------------------
def GetGOFrequencies( gene2go, genes ):
    """count number of each go category in gene list.

    return a tuple containing:
    * the total number of GO categories found.
    * dictionary of counts per GO category
    * dictionary of genes found with GO categories
    """
    counts = {}
    total = 0
    found_genes = {}
    
    for gene_id in genes:
        
        if gene_id not in gene2go: continue

        found_genes[gene_id] = 1
        for go in gene2go[gene_id]:
            if go.mGOId not in counts: counts[go.mGOId] = 0
            counts[go.mGOId] += 1
            total += 1

    return total, counts, found_genes

##------------------------------------------------------------------------------
def AnalyseGO( gene2go,
               genes,
               genes_background = (),
               do_probabilities = True):
    """analyse go ids.

    goids: list of goids to analyse
    genes: sample set of genes
    genes_background: background set of genes (default: all)
    """
    
    if genes_background == ():
        genes_background = gene2go.keys()

    result = GOResults()
    
    ## get background frequencies
    (background_counts_total, background_counts, background_genes) = \
        GetGOFrequencies( gene2go, 
                          genes_background )
    
    result.mBackgroundCountsTotal = background_counts_total
    result.mBackgroundNumCategories = len(background_counts)
    result.mBackgroundGenes = background_genes

    ## get sample frequencies
    (sample_counts_total, sample_counts, sample_genes) = \
                          GetGOFrequencies( gene2go, 
                                            genes )

    result.mNumGenes = len(genes)

    result.mSampleCountsTotal = sample_counts_total
    result.mSampleNumCategories = len(sample_counts)
    result.mSampleGenes = sample_genes
    
    # test for over or underrepresented categories in the slims
    # report results for all go categories in the background
    # so that also categories completely absent in the foreground (sample)
    # are considered.
    for go_id in background_counts.keys():
        
        result_go = GOResult(go_id)

        # use gene counts
        result_go.mSampleCountsCategory = sample_counts.get( go_id, 0 )
        result_go.mSampleCountsTotal = len(sample_genes)
        result_go.mBackgroundCountsTotal = len(background_genes)
        result_go.mBackgroundCountsCategory = background_counts[go_id]

        E.debug( "processing %s: genes in foreground=%i, genes in backgound=%i, sample_counts=%i, background_counts=%i" % \
                     ( go_id,
                       len(sample_genes),
                       len(background_genes),
                       sample_counts.get(go_id,0),
                       background_counts.get(go_id,0),
                       )
                 )

        if do_probabilities:
            try:
                result_go.UpdateProbabilities()
            except AssertionError, msg:
                print msg
                print "# error while calculating probabilities for %s" % go_id
                print "# genes in sample", sample_genes
                print "# counts in sample: %i out of %i total" % ( result_go.mSampleCountsCategory, result_go.mSampleCountsTotal)
                print "# counts in background %i out of %i total" % (result_go.mBackgroundCountsCategory, result_go.mBackgroundCountsTotal)
                for x in sample_genes.keys():
                    for y in gene2go[x]:
                        print x, str(y)

                sys.exit(0)
        
        result.mResults[go_id] = result_go
        
    return result

##--------------------------------------------------------------------------------
def GetGOStatement( go_type, database, species ):
    """build statement to get GO assignments for genes from ENSEMBL."""
    
    if database in ("ensembl_mart_27_1", ):
        statement = """SELECT DISTINCTROW
        gene_stable_id, glook_%s_id, description, olook_evidence_code
        FROM %s.%s_gene_ensembl__go_%s__look
        WHERE glook_%s_id IS NOT NULL
        GROUP BY gene_stable_id, glook_%s_id, description
        ORDER BY gene_stable_id
        """ % (go_type,
               database, species, go_type,
               go_type, go_type)
        
    elif database in ("ensembl_mart_31", "ensembl_mart_37", "ensembl_mart_41" ):
        statement = """SELECT DISTINCTROW
        gene_stable_id, glook_%s_id, description, olook_evidence_code
        FROM %s.%s_go_%s__go_%s__main
        WHERE glook_%s_id IS NOT NULL
        GROUP BY gene_stable_id, glook_%s_id, description
        ORDER BY gene_stable_id
        """ % (go_type,
               database, species, go_type, go_type,
               go_type, go_type)

    elif re.search( "core", database):

        if go_type == "biol_process":
            go_type = "biological_process"
        elif go_type == "mol_function":
            go_type = "molecular_function"
        elif go_type == "cell_location":
            go_type = "cellular_component"
        else:
            raise "unknown go_type %s" % go_type

        x = re.search("(\d+)", database)
        if not x:
            raise "can't find version number in database %s" % database

        version = int(x.groups()[0])
        if version <= 54:
            go_database = "ensembl_go_%s" % version
            go_field = "acc"
            statement = """SELECT DISTINCTROW
        g.stable_id, xref.dbprimary_acc, go.name, 'NA'
        FROM gene, transcript, translation, 
        gene_stable_id as g, object_xref as o, xref,
        %(go_database)s.term AS go
        WHERE gene.gene_id = transcript.gene_id
        AND transcript.transcript_id = translation.transcript_id
        AND g.gene_id = gene.gene_id
        AND translation.translation_id = o.ensembl_id
        AND xref.xref_id = o.xref_id
        AND go.%(go_field)s = xref.dbprimary_acc
        AND go.term_type = '%(go_type)s'
        AND xref.external_db_id = 1000
        """ % locals()

        elif version <=66:
            go_database = "ensembl_ontology_%s" % version
            go_field = "accession"

            statement = """SELECT DISTINCTROW
        g.stable_id, xref.dbprimary_acc, go.name, 'NA'
        FROM gene, transcript, translation, 
        gene_stable_id as g, object_xref as o, xref,
        %(go_database)s.term AS go,
        %(go_database)s.ontology AS ontology
        WHERE gene.gene_id = transcript.gene_id
        AND transcript.transcript_id = translation.transcript_id
        AND g.gene_id = gene.gene_id
        AND translation.translation_id = o.ensembl_id
        AND xref.xref_id = o.xref_id
        AND go.%(go_field)s = xref.dbprimary_acc
        AND go.ontology_id = ontology.ontology_id 
        AND ontology.namespace = '%(go_type)s'
        AND xref.external_db_id = 1000
        """ % locals()

        else:

            go_database = "ensembl_ontology_%s" % version
            go_field = "accession"

            statement = """SELECT DISTINCTROW
        gene.stable_id, xref.dbprimary_acc, go.name, 'NA'
        FROM gene, transcript, translation, 
        object_xref as o, xref,
        %(go_database)s.term AS go,
        %(go_database)s.ontology AS ontology
        WHERE gene.gene_id = transcript.gene_id
        AND transcript.transcript_id = translation.transcript_id
        AND translation.translation_id = o.ensembl_id
        AND xref.xref_id = o.xref_id
        AND go.%(go_field)s = xref.dbprimary_acc
        AND go.ontology_id = ontology.ontology_id 
        AND ontology.namespace = '%(go_type)s'
        AND xref.external_db_id = 1000
        """ % locals()
    else:
        raise "unknown ensmart version %s" % database

    return statement

##--------------------------------------------------------------------------------
def ReadGene2GOFromDatabase( dbhandle, go_type, database, species ):
    """read go assignments from ensembl database.

    returns a dictionary of lists.
    (one to many mapping of genes to GO categories)
    and a dictionary of go-term to go information

    Note: assumes that external_db_id for GO is 1000
    """

    statement = GetGOStatement( go_type, database, species )
    result = dbhandle.Execute(statement).fetchall()

    gene2go = {}
    go2info = collections.defaultdict( GOInfo )
    for gene_id, goid, description, evidence in result:
        gm = GOMatch( goid, go_type, description, evidence )
        gi = GOInfo( goid, go_type, description )
        if gene_id not in gene2go: gene2go[gene_id] = []
        gene2go[gene_id].append(gm)
        go2info[goid] = gi
    
    return gene2go, go2info

##---------------------------------------------------------------------------
def DumpGOFromDatabase( outfile, 
                        dbhandle, 
                        options ):

    """read go assignments from database.

    and dump them into a flatfile.
    (one to many mapping of genes to GO categories)
    and a dictionary of go-term to go information
    """

    E.info( "category\ttotal\tgenes\tcategories" )

    all_genes = collections.defaultdict( int )
    all_categories = collections.defaultdict( int )
    all_ntotal = 0

    outfile.write("go_type\tgene_id\tgo_id\tdescription\tevidence\n" )

    for go_type in options.ontology:

        genes = collections.defaultdict( int )
        categories = collections.defaultdict( int )
        ntotal = 0

        statement = GetGOStatement( go_type, options.database, options.species )
        
        results = dbhandle.Execute(statement).fetchall()

        for result in results:
            outfile.write( "\t".join(map( str, (go_type,) + result))+ "\n")
            gene_id, goid, description, evidence = result
            genes[gene_id] += 1
            categories[goid] += 1
            ntotal += 1
            all_genes[gene_id] += 1
            all_categories[goid] += 1
            all_ntotal += 1
            
        E.info("%s\t%i\t%i\t%i" % (go_type, ntotal,
                                   len(genes),
                                   len(categories) ))

    E.info( "%s\t%i\t%i\t%i" % ("all", 
                                all_ntotal,
                                len(all_genes),
                                len(all_categories) ) )

    return 

##---------------------------------------------------------------------------
def ReadGene2GOFromFile( infile, synonyms = {}, obsolete = {} ):
    """reads GO mappings for all go_types from a
    file.

    If synonyms is given, goids in synynoms will be translated.
    Terms in *obsolete* will be discarded.

    returns two maps: gene2go maps genes to go categories
    and go2info maps go categories to information.
    """

    gene2gos = {}
    go2infos = {}
    c = E.Counter()

    for line in infile:
        if line[0] == "#": continue
        try:
            go_type, gene_id, goid, description, evidence = line[:-1].split("\t")
        except ValueError, msg:
            raise ValueError("parsing error in line '%s': %s" % (line[:-1], msg))
        if go_type == "go_type": continue

        c.input += 1

        if goid in synonyms:
            c.synonyms += 1
            goid = synonyms[goid]

        if goid in obsolete: 
            c.obsolete += 1
            continue

        gm = GOMatch( goid, go_type, description, evidence )
        gi = GOInfo( goid, go_type, description )
        if go_type not in gene2gos:
            gene2gos[go_type] = {}
            go2infos[go_type] = {}

        gene2go = gene2gos[go_type]
        go2info = go2infos[go_type]
        
        if gene_id not in gene2go: gene2go[gene_id] = []
        gene2go[gene_id].append(gm)
        go2info[goid] = gi
        c.output += 1
        
    E.debug( "read gene2go assignments: %s" % str(c) )

    return gene2gos, go2infos

##---------------------------------------------------------------------------
def CountGO( gene2go ):
    """count number of genes and go categories in mapping."""

    cats = collections.defaultdict( int )
    nmaps = 0
    for k, vv in gene2go.items():
        for v in vv:
            nmaps += 1                    
            cats[v.mGOId] += 1
            
    return len(gene2go), len(cats), nmaps, cats

def removeCategories( gene2go, categories ):
    '''remove all genes that map to *categories*.'''
    
    for k, vv in gene2go.items():
        gene2go[k] = [ v for v in vv if v.mGOId not in categories ]

##---------------------------------------------------------------------------
def countGOs( gene2gos ):
    """return map of number of genes and go categories in mapping."""
    genes, goids = collections.defaultdict( int ), collections.defaultdict( int )

    for cat, gene2go in gene2gos.iteritems():
        for gene_id, vv in gene2go.iteritems():
            genes[gene_id] += 1
            for v in vv:
                goids[v.mGOId] += 1
    return genes, goids

##---------------------------------------------------------------------------
def ReadGeneLists( filename_genes, gene_pattern = None ):
    """read gene lists from filename in matrix.

    returns a tuple (list of all genes, dictionary of gene lists) 
    """

    if filename_genes == "-":
        infile = sys.stdin
    else:
        infile = IOTools.openFile(filename_genes,"r")

    headers, table = CSV.ReadTable( infile.readlines(), as_rows = False )

    if filename_genes != "-": infile.close()

    
    all_genes = table[0]
        
    # if there is only a single column, add a dummy column
    if len(table) == 1:
        table.append( [1] * len( table[0]) )
        headers.append( "foreground" )

    E.info( "read %i genes from %s" % (len(all_genes), filename_genes))

    if gene_pattern:
        rx = re.compile(gene_pattern)
        all_genes = map( lambda x: rx.search( x ).groups()[0], all_genes )

    gene_lists = collections.OrderedDict()
    for header, col in zip( headers[1:], table[1:]):
        s = list(set([x for x,y in zip(all_genes, col) if y != "0" ]))
        gene_lists[header] = set(s)

    return all_genes, gene_lists 

##---------------------------------------------------------------------------
def buildGO2Genes( gene2gos, ancestors = None ):
    '''invert the dictionary genes2go. 

    If ancestors is given, add missing ancestral information.
    '''
    
    go2genes = collections.defaultdict( set )
    for gene_id, terms in gene2gos.iteritems():
        for term in terms:
            go2genes[term.mGOId].add(gene_id)
            if ancestors:
                for anc in ancestors[term.mGOId]:
                    go2genes[anc].add(gene_id)
    return go2genes

##---------------------------------------------------------------------------
def GetCode( v ):
    """return a code for over/underrepresentation."""

    if v.mRatio > 1.0:
        code = "+"
    elif v.mRatio < 1.0:
        code = "-"
    else:
        code = "?"
    return code

##---------------------------------------------------------------------------    
def convertGo2Goslim( options ):
    """read gene list with GO assignments and convert to GO slim categories."""
    
    E.info( "reading GO assignments from stdin" )
    gene2gos, go2infos = ReadGene2GOFromFile( options.stdin )
    input_genes, input_goids = countGOs( gene2gos )

    #############################################################
    ## read GO ontology from file
    assert options.filename_ontology, "please supply a GO ontology"
    E.info( "reading ontology from %s" % (options.filename_ontology) )
        
    infile = IOTools.openFile(options.filename_ontology)
    ontology = readOntology( infile )
    infile.close()
        
    go2infos = collections.defaultdict( dict )
    # substitute go2infos
    for go in ontology.values():
        go2infos[go.mNameSpace][go.mId] = GOInfo( go.mId,
                                                  go_type = go.mNameSpace,
                                                  description = go.mName )

    E.info( "reading GO assignments from %s" % options.filename_slims)
    go_slims = GetGOSlims( IOTools.openFile(options.filename_slims, "r") )

    if options.loglevel >=1:
        v = set()
        for x in go_slims.values():
            for xx in x: v.add(xx)
        E.info( "read go slims from %s: go=%i, slim=%i" %\
                                  ( options.filename_slims,
                                    len(go_slims), 
                                    len( v ) ))
                
    output_goids, output_genes = set(), set()
    noutput = 0
    options.stdout.write( "\t".join( ("go_type", "gene_id", "go_id", "description", "evidence" ) ) + "\n" )
    for category, gene2go in gene2gos.iteritems():
        gene2go = MapGO2Slims( gene2go, go_slims, ontology )
        for gene_id, values in gene2go.iteritems():
            output_genes.add( gene_id )
            for go in values:
                output_goids.add( go.mGOId)
                options.stdout.write( "%s\t%s\t%s\t%s\t%s\n" % \
                                          ( go.mGOType,
                                            gene_id,
                                            go.mGOId,
                                            go.mDescription,
                                            "NA", ) )
                noutput += 1

    E.info( "ninput_genes=%i, ninput_goids=%i, noutput_gene=%i, noutput_goids=%i, noutput=%i" % \
                         (len(input_genes), len(input_goids),
                          len(output_genes), len(output_goids),
                          noutput) )

def outputResults( outfile, 
                   pairs, 
                   go2info,
                   options,
                   fdrs = None, 
                   samples = None, 
                   gene2go = None, 
                   foreground = None,
                   gene2name = None):
    '''output GO results to outfile.

    If foreground is given, output a list of gene identifiers in the foreground.
    If gene2name is given, output a columns with gene names (instead of identifiers)
    '''

    headers = ["code",
               "scount", "stotal", "spercent", 
               "bcount", "btotal", "bpercent", 
               "ratio",
               "pvalue", "pover", "punder", 
               "goid", "category", "description"]

    if fdrs:
        headers += ["fdr"]

    if gene2go and foreground:
        headers += ['foreground']
        go2genes = buildGO2Genes( gene2go )
        if gene2name:
            headers += ['genes']

    if samples:
        headers += ["min", "max", "zscore", "mpover", "mpunder", 
                    "nfdr_expected",
                    "CI95lower", "CI95upper" ]

    outfile.write("\t".join(headers) + "\n" )

    nselected = 0

    for k, v in pairs:

        code = GetCode( v )

        n = go2info.get( k, GOInfo() )

        outfile.write("%s\t%s\t%s" % (code, str(v), n))
        
        if options.fdr:
            fdr = fdrs[k][0]
            outfile.write( "\t%f" % fdr )

        if options.sample:

            if k in samples:
                s = samples[k]

            ## calculate values for z-score
            if s.mStddev > 0:
                zscore = abs(float(v.mSampleCountsCategory) - s.mMean) / s.mStddev
            else:
                zscore = 0.0

            outfile.write("\t%i\t%i\t%f\t%5.2e\t%5.2e\t%6.4f\t%6.4f\t%6.4f" %\
                          (s.mMin,
                           s.mMax,
                           zscore,
                           min(s.mProbabilitiesOverRepresentation),
                           min(s.mProbabilitiesUnderRepresentation),
                           scipy.mean( s.mCounts ),
                           scipy.stats.scoreatpercentile( s.mCounts, 5 ),
                           scipy.stats.scoreatpercentile( s.mCounts, 95 ),
                           ) )
        
        if foreground:
            if k in go2genes:
                g = [ x for x in go2genes[k] if x in foreground]
                if gene2name:
                    g = [gene2name.get(x,'?') for x in g ]
                g = ";".join( g )
            else:
                g = ""
            outfile.write("\t%s" % g )

        outfile.write("\n")

def getSamples( gene2go, genes, background, options, test_ontology ):

    sample_size = options.sample
    # List of all minimum probabilities in simulation
    simulation_min_pvalues = []
    E.info( "sampling: calculating %i samples: " % (sample_size))

    counts = {}
    prob_overs = {}
    prob_unders = {}

    samples = {}

    options.stdlog.write("# ")
    options.stdlog.flush()

    for x in range(sample_size):

        if options.loglevel >= 1:
            options.stdlog.write( "." )
            options.stdlog.flush()

        ## get shuffled array of genes from background
        sample_genes = random.sample( background, len(genes) )

        go_results = AnalyseGO( gene2go , sample_genes, background )

        pairs = go_results.mResults.items()

        for k, v in pairs:
            if k not in counts:
                counts[k] = []
                prob_overs[k] = []
                prob_unders[k] = []

            counts[k].append( v.mSampleCountsCategory )
            prob_overs[k].append( v.mProbabilityOverRepresentation )
            prob_unders[k].append( v.mProbabilityUnderRepresentation )                    

            simulation_min_pvalues.append( self.mPValue )

    if options.loglevel >= 1:
        sys.stdout.write("\n")
        sys.stdout.flush()

    E.info("sampling: sorting %i P-Values" % len(simulation_min_pvalues) )

    simulation_min_pvalues.sort()
    simulation_min_pvalues = numpy.array(simulation_min_pvalues)

    samples = {}

    if options.output_filename_pattern:
        filename = options.output_filename_pattern % { 'go': test_ontology, 'section': "samples" }
        E.info( "sampling results go to %s" % filename )
        outfile = IOTools.openFile( filename, "w", create_dir = True )
    else:
        outfile = sys.stdout

    outfile.write( "\t".join( ("goid", "min", "max", "mean", "median", "stddev", 
                               "CI95lower", "CI95upper",
                               "pover", "punder", "goid",
                               "category", "description") ) + "\n" )
    for k in counts.keys():

        c = counts[k]

        prob_overs[k].sort()
        prob_unders[k].sort()

        s = GOSample(min(c),
                     max(c),
                     scipy.mean(c),
                     numpy.std(c),
                     numpy.array(prob_overs[k]),
                     numpy.array(prob_unders[k]),
                     counts[k] )

        samples[k] = s

        outfile.write( "%s\t%i\t%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%s\n" %\
                       (k,
                        min(c),
                        max(c),
                        scipy.mean(c),
                        scipy.median(c),
                        numpy.std(c),
                        scipy.stats.scoreatpercentile( c, 5 ),
                        scipy.stats.scoreatpercentile( c, 95 ),
                        min(prob_overs[k]),
                        min(prob_unders[k]),
                        go2info[k] ))

    if options.output_filename_pattern:
        outfile.close()

    return samples, simulation_min_pvalues

def computeFDRs( go_results, options, test_ontology ):

    pairs = go_results.mResults.items()

    E.info( "calculating the FDRs using method `%s`" % options.qvalue_method )

    samples = None

    observed_min_pvalues = [ min(x[1].mProbabilityOverRepresentation,
                                 x[1].mProbabilityUnderRepresentation) for x in pairs ]

    fdrs = {}

    method = options.qvalue_method

    if options.qvalue_method == "storey":

        # compute fdr via Storey's method
        try:
            fdr_data = Stats.doFDR( observed_min_pvalues )

        except ValueError, msg:
            E.warn( "failure in q-value computation: %s" % msg )
            E.warn( "reverting to Bonferroni correction" )
            method = "bonf"
            fdr_data = Stats.FDRResult()
            l = float(len(observed_min_pvalues))
            fdr_data.mQValues = [ min(1.0, x * l) for x in observed_min_pvalues ]

        for pair, qvalue in zip( pairs, fdr_data.mQValues ):
            fdrs[pair[0]] = (qvalue, 1.0, 1.0)

    elif options.qvalue_method == "empirical":
        assert options.sample > 0, "requiring a sample size of > 0"

        #############################################################################
        ## sampling
        ## for each GO-category:
        ##      get maximum and minimum counts in x samples -> calculate minimum/maximum significance
        ##      get average and stdev counts in x samples -> calculate z-scores for test set
        samples, simulation_min_pvalues = getSamples( gene2go, genes, background, options, test_ontology )

        # compute P-values from sampling
        observed_min_pvalues.sort()
        observed_min_pvalues = numpy.array( observed_min_pvalues )

        sample_size = options.sample

        for k, v in pairs:

            if k in samples:
                s = samples[k]
            else:
                raise KeyError("category %s not in samples" % k)

            ## calculate values for z-score
            if s.mStddev > 0:
                zscore = abs(float(v.mSampleCountsCategory) - s.mMean) / s.mStddev
            else:
                zscore = 0.0

            #############################################################
            # FDR:
            # For each p-Value p at node n:
            #   a = average number of nodes in each simulation run with P-Value < p
            #           this can be obtained from the array of all p-values and all nodes
            #           simply divided by the number of samples.
            #      aka: expfpos=experimental false positive rate
            #   b = number of nodes in observed data, that have a P-Value of less than p.
            #      aka: pos=positives in observed data
            #   fdr = a/b
            pvalue = v.mPValue

            # calculate values for FDR: 
            # nfdr = number of entries with P-Value better than node.
            a = 0
            while a < len(simulation_min_pvalues) and \
                      simulation_min_pvalues[a] < pvalue:
                a += 1
            a = float(a) / float(sample_size)
            b = 0
            while b < len(observed_min_pvalues) and \
                    observed_min_pvalues[b] < pvalue:
                b += 1

            if b > 0:
                fdr = min(1.0, float(a) / float(b))
            else:
                fdr = 1.0

            fdrs[k] = (fdr, a, b)
    else:
        qvalues = R['p.adjust']( observed_min_pvalues, method = options.qvalue_method )
        fdr_data = Stats.FDRResult()
        fdr_data.mQValues = list(qvalues)
        for pair, qvalue in zip( pairs, fdr_data.mQValues ):
            fdrs[pair[0]] = (qvalue, 1.0, 1.0)
        
    return fdrs, samples, method

################################################################
################################################################
################################################################
def getFileName( options, **kwargs ):
    '''return a filename

    Placeholders in filename are string-substituted with the 
    dictionary in kwargs.
    '''
    if options.output_filename_pattern:
        filename = options.output_filename_pattern % kwargs
        E.info( "output for section '%s' go to %s" % (kwargs.get("section", "unknown"), filename))
        outfile = IOTools.openFile(filename, "w", create_dir = True)
    else:
        outfile = options.stdout

    return outfile

################################################################
################################################################
################################################################
def buildMatrix( results, valuef, dtype = numpy.float, default = 0 ):
    '''build a matrix from a field in *results*

    The value stored in the matrix is accessed via *valuef*.
    '''

    row_headers = [ set( [x[0] for x in y ] ) for y in results ]
    row_headers = sorted( list( row_headers[0].union( *row_headers[1:] ) ) )
    map_row = dict( zip(row_headers, range(len(row_headers)) ) )
    matrix = numpy.zeros( (len(row_headers), len(results) ), dtype = dtype )
    if default != 0: matrix[:] = default

    for col, pairs in enumerate(results):
        for row,v in pairs: 
            matrix[map_row[row]][col] = valuef( v )

    return matrix, row_headers

################################################################
################################################################
################################################################
def selectSignificantResults( pairs, fdrs, options ):
    '''select a set of significant results.
    '''

    filtered_pairs = []
    for k, v in pairs:

        is_ok = False

        pvalue = v.mPValue

        if options.fdr:
            (fdr, expfpos, pos) = fdrs[k]
            if fdr < options.threshold: is_ok = True
        else:
            if pvalue < options.threshold: is_ok = True

        if is_ok: filtered_pairs.append( (k,v) )

    return filtered_pairs

################################################################
################################################################
################################################################
def outputMultipleGeneListResults( results, 
                                   all_genelists_with_results, 
                                   test_ontology,
                                   go2info,
                                   options,
                                   section ):
    '''select a set of significant results.
    '''
    
    col_headers = all_genelists_with_results

    assert len(col_headers) == len(results)

    def _output( section, subsection, valuef, dtype ):

        # fold change matrix
        matrix, row_headers = buildMatrix( results, 
                                           valuef = valuef,
                                           dtype = dtype )

        outfile = getFileName( options, 
                               go = test_ontology,
                               section = section,
                               set = '%s_all' % subsection )

        IOTools.writeMatrix( outfile, matrix, row_headers, col_headers, row_header="category" )

        outfile = getFileName( options, 
                               go = test_ontology,
                               section = section,
                               set = '%s_alldesc' % subsection )

        IOTools.writeMatrix( outfile, matrix, 
                             [ "%s:%s" % (x, go2info[x].mDescription) for x in row_headers], 
                             col_headers, row_header="category" )


    _output( 'l2fold', section, 
             valuef = lambda x: math.log( x.mRatio + 0.00000001, 2 ),
             dtype = numpy.float )

    _output( 'l10pvalue', section, 
             valuef = lambda x: int(-10*math.log( x.mPValue,10)),
             dtype = numpy.int )

    _output( 'l10qvalue', section, 
             valuef = lambda x: int(-10*math.log( x.mQValue,10)),
             dtype = numpy.int )
             

def pairwiseGOEnrichment( results_per_genelist, labels, test_ontology, go2info, options ):
    '''compute pairwise enrichment between sets.

    The purpose of this method is to find if there are categories that are differently enriched
    in a pair of gene lists.
     
    The appropriate test here is the Chi-Squared test. 

    The assumption is that the background set is the same in all gene lists.

    The workflow is thus::

       for each combination of two gene lists:
           for each GO category:
               get counts in foreground, total counts of foreground
               compute chi-square enrichment output
               save P-value
           apply fdr - output significant differences.
    '''
    
    dicts = [ dict(x) for x in results_per_genelist ]

    PairResult = collections.namedtuple( "PairResult",
                                         "goid set1 set2 counts1 total1 pvalue1 qvalue1 counts2 total2 pvalue2 qvalue2 pvalue qvalue description" )

    outfile = getFileName( options, 
                           go = test_ontology,
                           section = 'summary',
                           set = "pairs" )

    outfile.write( "set1\tset2\ttotal1\ttotal2\tshared\tskipped\ttested\tsignificant\tinsignificant\n" )

    results = []

    total = len(dicts) * (len(dicts) -1) / 2 
    
    iteration = 0

    min_observed_counts = options.pairs_min_observed_counts

    for x, genelist1 in enumerate(dicts):
        

        x_go_categories = set( genelist1.keys() )
        for y, genelist2 in enumerate( dicts[:x] ):

            iteration += 1
            if iteration % 10 == 0:
                E.info( "iteration: %i/%i (%5.2f%%)" % (iteration, total, 100.0 * iteration / total) )
                
            y_go_categories = set(genelist2.keys())
            
            shared = x_go_categories.intersection( y_go_categories )
            
            c = E.Counter()

            for category in shared:
                c.shared += 1
                xx = genelist1[category]
                yy = genelist2[category]

                # discard all tests with few observations in the observed counts
                if xx.mSampleCountsCategory < min_observed_counts and yy.mSampleCountsCategory < min_observed_counts:
                    c.skipped += 1
                    continue
                
                observed = (xx.mSampleCountsCategory, yy.mSampleCountsCategory)

                aa, bb, cc, dd = \
                    (xx.mSampleCountsCategory,
                     yy.mSampleCountsCategory,
                     xx.mSampleCountsTotal - xx.mSampleCountsCategory,
                     yy.mSampleCountsTotal - yy.mSampleCountsCategory )

                if cc == dd == 0:
                    c.skipped += 1
                    continue

                c.tested += 1

                fisher, pvalue = scipy.stats.fisher_exact( numpy.array( 
                        ((aa, bb),
                         (cc, dd))))
                               
                if pvalue < 0.05:
                    c.significant_pvalue += 1
                else:
                    c.insignificant_pvalue += 1

                results.append( PairResult._make( (category,
                                                   labels[x],
                                                   labels[y],
                                                   xx.mSampleCountsCategory, 
                                                   xx.mSampleCountsTotal,
                                                   xx.mPValue,
                                                   xx.mQValue,
                                                   yy.mSampleCountsCategory,
                                                   yy.mSampleCountsTotal,
                                                   yy.mPValue,
                                                   yy.mQValue,
                                                   pvalue,
                                                   1.0,
                                                   go2info[category].mDescription ) ) )

            outfile.write( "\t".join( map(str,
                                          (labels[x], labels[y],
                                           len(x_go_categories),
                                           len(y_go_categories),
                                           c.shared,
                                           c.skipped,
                                           c.tested,
                                           c.significant_pvalue,
                                           c.insignicant_pvalue) ) ) + "\n" )
    if options.output_filename_pattern:
        outfile.close()

    if options.fdr:
        pvalues = [x.pvalue for x in results ]
        
        if options.qvalue_method == "storey":

            # compute fdr via Storey's method
            try:
                fdr_data = Stats.doFDR( pvalues )

            except ValueError, msg:
                E.warn( "failure in q-value computation: %s" % msg )
                E.warn( "reverting to Bonferroni correction" )
                method = "bonf"
                fdr_data = Stats.FDRResult()
                l = float(len(pvalues))
                fdr_data.mQValues = [ min(1.0, x * l) for x in pvalues ]

            qvalues = fdr_data.mQValues 
        else:
            qvalues = R['p.adjust']( pvalues, method = options.qvalue_method )

        # update qvalues
        results = [ x._replace( qvalue = y ) for x,y in zip( results, qvalues ) ]

    outfile = getFileName( options, 
                           go = test_ontology,
                           section = 'pairs',
                           set = "pairs" )
    
    outfile.write( "\t".join( PairResult._fields) + "\n" )
    for result in results:
        outfile.write( "\t".join( map(str, result ) )+ "\n" )
    
    if options.output_filename_pattern:
        outfile.close()
    
